# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from designateclient.cli import base

LOG = logging.getLogger(__name__)


class TouchDomainCommand(base.DeleteCommand):
    """Touch a single Domain"""

    def get_parser(self, prog_name):
        parser = super(TouchDomainCommand, self).get_parser(prog_name)

        parser.add_argument('domain_id', help="Domain ID")

        return parser

    def execute(self, parsed_args):
        self.client.touch.domain(parsed_args.domain_id)

        LOG.info('Domain touched successfully')



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from debtcollector import removals
from stevedore import extension

from designateclient import exceptions
from designateclient import utils
from designateclient import version


@removals.removed_class(
    'designateclient.v1.Client',
    replacement='designateclient.v2.client.Client',
    message='Designate v1 API is being retired, and the v1 Client class will '
    'stop functioning. Please update code to the '
    'designateclient.v2.client.Client class. The API is deprecated',
    version='2.2.0',
    removal_version='?',
    stacklevel=3)
class Client(object):
    """Client for the Designate v1 API"""

    def __init__(self, endpoint=None, username=None, user_id=None,
                 user_domain_id=None, user_domain_name=None, password=None,
                 tenant_name=None, tenant_id=None, domain_name=None,
                 domain_id=None, project_name=None,
                 project_id=None, project_domain_name=None,
                 project_domain_id=None, auth_url=None, token=None,
                 endpoint_type='publicURL', region_name=None,
                 service_type='dns', insecure=False, session=None,
                 cacert=None, all_tenants=None, edit_managed=None,
                 timeout=None):
        """
        :param endpoint: Endpoint URL
        :param token: A token instead of username / password
        :param insecure: Allow "insecure" HTTPS requests
        """

        if endpoint:
            endpoint = endpoint.rstrip('/')
            if not endpoint.endswith('v1'):
                endpoint = "%s/v1" % endpoint

        # Compatibility code to mimic the old behaviour of the client
        if session is None:
            session = utils.get_session(
                auth_url=auth_url,
                endpoint=endpoint,
                domain_id=domain_id,
                domain_name=domain_name,
                project_id=project_id or tenant_id,
                project_name=project_name or tenant_name,
                project_domain_name=project_domain_name,
                project_domain_id=project_domain_id,
                username=username,
                user_id=user_id,
                password=password,
                user_domain_id=user_domain_id,
                user_domain_name=user_domain_name,
                token=token,
                insecure=insecure,
                cacert=cacert,
            )

        # NOTE: all_tenants and edit_managed are pulled from the session for
        #       backwards compat reasons, do not pull additional modifiers from
        #       here. Once removed, the kwargs above should default to False.
        if all_tenants is None:
            self.all_tenants = getattr(session, 'all_tenants', False)
        else:
            self.all_tenants = all_tenants

        if edit_managed is None:
            self.edit_managed = getattr(session, 'edit_managed', False)
        else:
            self.edit_managed = edit_managed

        # Since we have to behave nicely like a legacy client/bindings we use
        # an adapter around the session to not modify it's state.
        interface = endpoint_type.rstrip('URL')

        self.session = utils.AdapterWithTimeout(
            session,
            auth=session.auth,
            endpoint_override=endpoint,
            region_name=region_name,
            service_type=service_type,
            interface=interface,
            user_agent='python-designateclient-%s' % version.version_info,
            version='1',
            timeout=timeout,
        )

        def _load_controller(ext):
            controller = ext.plugin(client=self)
            setattr(self, ext.name, controller)

        # Load all controllers
        mgr = extension.ExtensionManager('designateclient.v1.controllers')
        mgr.map(_load_controller)

    def wrap_api_call(self, func, *args, **kw):
        """
        Wrap a self.<rest function> with exception handling

        :param func: The function to wrap
        """
        kw['raise_exc'] = False
        kw.setdefault('headers', {})
        kw['headers'].setdefault('Content-Type', 'application/json')
        if self.all_tenants:
            kw['headers'].update({'X-Auth-All-Projects': 'true'})
        if self.edit_managed:
            kw['headers'].update({'X-Designate-Edit-Managed-Records': 'true'})

        # Trigger the request
        response = func(*args, **kw)

        # Decode is response, if possible
        try:
            response_payload = response.json()
        except ValueError:
            response_payload = {}

        if response.status_code == 400:
            raise exceptions.BadRequest(**response_payload)
        elif response.status_code in (401, 403, 413):
            raise exceptions.Forbidden(**response_payload)
        elif response.status_code == 404:
            raise exceptions.NotFound(**response_payload)
        elif response.status_code == 409:
            raise exceptions.Conflict(**response_payload)
        elif response.status_code >= 500:
            raise exceptions.Unknown(**response_payload)
        else:
            return response

    def get(self, path, **kw):
        return self.wrap_api_call(self.session.get, path, **kw)

    def post(self, path, **kw):
        return self.wrap_api_call(self.session.post, path, **kw)

    def put(self, path, **kw):
        return self.wrap_api_call(self.session.put, path, **kw)

    def delete(self, path, **kw):
        return self.wrap_api_call(self.session.delete, path, **kw)



"""
Copyright 2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from tempest.lib.exceptions import CommandFailed

from designateclient.functionaltests.base import BaseDesignateTest
from designateclient.functionaltests.datagen import random_a_recordset_name
from designateclient.functionaltests.datagen import random_zone_name
from designateclient.functionaltests.v2.fixtures import RecordsetFixture
from designateclient.functionaltests.v2.fixtures import ZoneFixture


class TestRecordset(BaseDesignateTest):

    def setUp(self):
        super(TestRecordset, self).setUp()
        self.ensure_tld_exists('com')
        self.zone = self.useFixture(ZoneFixture(
            name=random_zone_name(),
            email='test@example.com',
        )).zone

        name = random_a_recordset_name(self.zone.name)
        self.recordset = self.useFixture(RecordsetFixture(
            zone_id=self.zone.id,
            name=name,
            records='1.2.3.4',
            description='An a recordset',
            type='A',
            ttl=1234,
        )).recordset

        self.assertEqual(self.recordset.name, name)
        self.assertEqual(self.recordset.records, '1.2.3.4')
        self.assertEqual(self.recordset.description, 'An a recordset')
        self.assertEqual(self.recordset.type, 'A')
        self.assertEqual(self.recordset.ttl, '1234')

    def test_recordset_list(self):
        rsets = self.clients.recordset_list(self.zone.id)
        self.assertGreater(len(rsets), 0)

    def test_recordset_create_and_show(self):
        rset = self.clients.recordset_show(self.zone.id, self.recordset.id)
        self.assertTrue(hasattr(self.recordset, 'action'))
        self.assertTrue(hasattr(rset, 'action'))
        self.assertEqual(self.recordset.created_at, rset.created_at)
        self.assertEqual(self.recordset.description, rset.description)
        self.assertEqual(self.recordset.id, rset.id)
        self.assertEqual(self.recordset.name, rset.name)
        self.assertEqual(self.recordset.records, rset.records)
        self.assertEqual(self.recordset.status, rset.status)
        self.assertEqual(self.recordset.ttl, rset.ttl)
        self.assertEqual(self.recordset.type, rset.type)
        self.assertEqual(self.recordset.updated_at, rset.updated_at)
        self.assertEqual(self.recordset.version, rset.version)
        self.assertEqual(self.recordset.zone_id, self.zone.id)

    def test_recordset_delete(self):
        rset = self.clients.recordset_delete(self.zone.id, self.recordset.id)
        self.assertEqual(rset.action, 'DELETE')
        self.assertEqual(rset.status, 'PENDING')

    def test_recordset_set(self):
        rset = self.clients.recordset_set(
            self.zone.id,
            self.recordset.id,
            records='2.3.4.5',
            ttl=2345,
            description='Updated description',
        )

        self.assertEqual(rset.records, '2.3.4.5')
        self.assertEqual(rset.ttl, '2345')
        self.assertEqual(rset.description, 'Updated description')

    def test_recordset_set_clear_ttl_and_description(self):
        rset = self.clients.recordset_set(
            self.zone.id,
            self.recordset.id,
            no_description=True,
            no_ttl=True,
        )

        self.assertEqual(rset.description, 'None')
        self.assertEqual(rset.ttl, 'None')


class TestRecordsetNegative(BaseDesignateTest):

    def test_invalid_option_on_recordset_create(self):
        cmd = 'recordset create de47d30b-41c5-4e38-b2c5-e0b908e19ec7 ' \
            'aaa.desig.com. --type A --records 1.2.3.4 ' \
            '--invalid "not valid"'
        self.assertRaises(CommandFailed, self.clients.openstack, cmd)

    def test_invalid_recordset_command(self):
        cmd = 'recordset hopefullynotvalid'
        self.assertRaises(CommandFailed, self.clients.openstack, cmd)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from mock import patch

from designateclient.tests import test_v1
from designateclient.v1 import sync


class TestSync(test_v1.APIV1TestCase, test_v1.CrudMixin):

    @patch.object(sync.SyncController, "sync_all")
    def test_sync_all(self, sync_all):
        self.client.sync.sync_all()
        self.client.sync.sync_all.assert_called_with()

    @patch.object(sync.SyncController, "sync_domain")
    def test_sync_domain(self, sync_domain):
        args = mock.MagicMock()
        args.tenant_id = "1234"
        self.client.sync.sync_domain(args.tenant_id)
        self.client.sync.sync_domain.assert_called_with("1234")

    @patch.object(sync.SyncController, "sync_record")
    def test_sync_record(self, sync_record):
        args = mock.MagicMock()
        args.tenant_id = "1234"
        args.record_id = "uuid"
        self.client.sync.sync_record(args.tenant_id, args.record_id)
        self.client.sync.sync_record.assert_called_with("1234", "uuid")



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

import mock
from mock import patch

from designateclient.tests import test_v1
from designateclient.v1 import servers


class TestServers(test_v1.APIV1TestCase, test_v1.CrudMixin):
    RESOURCE = 'servers'

    def new_ref(self, **kwargs):
        ref = super(TestServers, self).new_ref(**kwargs)
        ref.setdefault("name", uuid.uuid4().hex)
        return ref

    def test_list(self):
        items = [
            self.new_ref(name="ns1.example.org.",
                         id="89acac79-38e7-497d-807c-a011e1310438"),
            self.new_ref(name="ns2.example.org.",
                         id="89acac79-38e7-497d-807c-a011e1310435")
        ]

        self.stub_url("GET", parts=[self.RESOURCE], json={"servers": items})

        listed = self.client.servers.list()
        self.assertList(items, listed)
        self.assertQueryStringIs("")

    def test_get(self):
        ref = self.new_ref(name="ns1.example.org.",
                           id="89acac79-38e7-497d-807c-a011e1310438")

        self.stub_entity("GET", entity=ref, id=ref["id"])

        response = self.client.servers.get(ref["id"])
        self.assertEqual(ref, response)

    def test_create(self):
        ref = {"id": "89acac79-38e7-497d-807c-a011e1310438",
               "name": "ns1.example.org."}

        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.client.servers.create({"name": "ns1.example.org."})
        self.assertRequestBodyIs(json=values)

    def test_create_with_name_too_long(self):
        ref = {"id": "89acac79-38e7-497d-807c-a011e1310438",
               "name": "ns1." + "foo" * 85 + ".org."}

        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.assertRaises(ValueError, self.client.servers.create,
                          {"name": "ns1.example.org."})

    @patch.object(servers.ServersController, "update")
    def test_update(self, server_update):
        ref = self.new_ref()

        self.stub_entity("PUT", entity=ref, id=ref["id"])

        mock_server = mock.MagicMock()
        self.client.servers.update(mock_server)
        self.client.servers.update.assert_called_with(mock_server)

    def test_delete(self):
        ref = self.new_ref()

        self.stub_entity("DELETE", id=ref["id"])

        self.client.servers.delete(ref["id"])
        self.assertRequestBodyIs(None)



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from osc_lib.command import command
from osc_lib import exceptions as osc_exc
import six

from designateclient import utils
from designateclient.v2.cli import common
from designateclient.v2.utils import get_all


LOG = logging.getLogger(__name__)


def _format_zone(zone):
    zone.pop('links', None)
    zone['masters'] = ", ".join(zone['masters'])
    attrib = ''
    for attr in zone['attributes']:
        attrib += "%s:%s\n" % (attr, zone['attributes'][attr])
    zone['attributes'] = attrib


def _format_zone_export_record(zone_export_record):
    zone_export_record.pop('links', None)


def _format_zone_import_record(zone_import_record):
    zone_import_record.pop('links', None)


class ListZonesCommand(command.Lister):
    """List zones"""

    columns = ['id', 'name', 'type', 'serial', 'status', 'action']

    def get_parser(self, prog_name):
        parser = super(ListZonesCommand, self).get_parser(prog_name)

        parser.add_argument('--name', help="Zone Name", required=False)
        parser.add_argument('--email', help="Zone Email", required=False)
        parser.add_argument('--type', help="Zone Type", required=False)
        parser.add_argument('--ttl', help="Time To Live (Seconds)",
                            required=False)
        parser.add_argument('--description', help="Description",
                            required=False)
        parser.add_argument('--status', help="Zone Status", required=False)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        criterion = {}
        if parsed_args.type is not None:
            criterion["type"] = parsed_args.type

        if parsed_args.name is not None:
            criterion["name"] = parsed_args.name

        if parsed_args.ttl is not None:
            criterion["ttl"] = parsed_args.ttl

        if parsed_args.description is not None:
            criterion["description"] = parsed_args.description

        if parsed_args.email is not None:
            criterion["email"] = parsed_args.email

        if parsed_args.status is not None:
            criterion["status"] = parsed_args.status

        data = get_all(client.zones.list, criterion)

        cols = self.columns

        if client.session.all_projects:
            cols.insert(1, 'project_id')

        return cols, (utils.get_item_properties(s, cols) for s in data)


class ShowZoneCommand(command.ShowOne):
    """Show zone details"""

    def get_parser(self, prog_name):
        parser = super(ShowZoneCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Zone ID")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zones.get(parsed_args.id)

        _format_zone(data)
        return six.moves.zip(*sorted(six.iteritems(data)))


class CreateZoneCommand(command.ShowOne):
    """Create new zone"""

    def get_parser(self, prog_name):
        parser = super(CreateZoneCommand, self).get_parser(prog_name)

        parser.add_argument('name', help="Zone Name")
        parser.add_argument('--email', help="Zone Email")
        parser.add_argument('--type', help="Zone Type", default='PRIMARY')
        parser.add_argument('--ttl', type=int, help="Time To Live (Seconds)")
        parser.add_argument('--description', help="Description")
        parser.add_argument('--masters', help="Zone Masters", nargs='+')
        parser.add_argument('--attributes', help="Zone Attributes", nargs='+')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        payload = {}

        if parsed_args.description:
            payload["description"] = parsed_args.description

        if parsed_args.attributes:
            payload["attributes"] = {}
            for attr in parsed_args.attributes:
                try:
                    k, v = attr.split(':')
                    payload["attributes"][k] = v
                except ValueError:
                    msg = "Attribute '%s' is in an incorrect format. "\
                          "Attributes are <key>:<value> formated"
                    raise osc_exc.CommandError(msg % attr)

        if parsed_args.type == 'PRIMARY':
            # email is just for PRIMARY.
            if not parsed_args.email:
                msg = "Zone type PRIMARY requires --email."
                raise osc_exc.CommandError(msg)

            payload["email"] = parsed_args.email

            # TTL is just valid for PRIMARY
            if parsed_args.ttl is not None:
                payload["ttl"] = parsed_args.ttl
        elif parsed_args.type == 'SECONDARY':
            payload["masters"] = parsed_args.masters
        else:
            msg = "Type %s is not supported. Please choose between " \
                "PRIMARY or SECONDARY"
            raise osc_exc.CommandError(msg % parsed_args.type)

        data = client.zones.create(
            parsed_args.name, parsed_args.type, **payload)

        _format_zone(data)
        return six.moves.zip(*sorted(six.iteritems(data)))


class SetZoneCommand(command.ShowOne):
    """Set zone properties"""

    def get_parser(self, prog_name):
        parser = super(SetZoneCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Zone ID")
        parser.add_argument('--email', help="Zone Email")
        parser.add_argument('--ttl', type=int, help="Time To Live (Seconds)")
        description_group = parser.add_mutually_exclusive_group()
        description_group.add_argument('--description', help="Description")
        description_group.add_argument('--no-description', action='store_true')

        parser.add_argument('--masters', help="Zone Masters", nargs='+')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = {}

        # TODO(kiall): API needs updating.. this get is silly
        if parsed_args.email:
            data['email'] = parsed_args.email

        if parsed_args.ttl:
            data['ttl'] = parsed_args.ttl

        if parsed_args.no_description:
            data['description'] = None
        elif parsed_args.description:
            data['description'] = parsed_args.description

        if parsed_args.masters:
            data['masters'] = parsed_args.masters

        updated = client.zones.update(parsed_args.id, data)
        _format_zone(updated)
        return six.moves.zip(*sorted(six.iteritems(updated)))


class DeleteZoneCommand(command.ShowOne):
    """Delete zone"""

    def get_parser(self, prog_name):
        parser = super(DeleteZoneCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Zone ID")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zones.delete(parsed_args.id)
        LOG.info('Zone %s was deleted', parsed_args.id)

        _format_zone(data)
        return six.moves.zip(*sorted(six.iteritems(data)))


class AbandonZoneCommand(command.Command):
    """Abandon a zone"""
    def get_parser(self, prog_name):
        parser = super(AbandonZoneCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Zone ID")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        client.zones.abandon(parsed_args.id)

        LOG.info("Z %(zone_id)s abandoned",
                 {"zone_id": parsed_args.id})


class AXFRZoneCommand(command.Command):
    """AXFR a zone"""
    def get_parser(self, prog_name):
        parser = super(AXFRZoneCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Zone ID")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        client.zones.axfr(parsed_args.id)

        LOG.info("Scheduled AXFR for zone %(zone_id)s",
                 {"zone_id": parsed_args.id})


class CreateTransferRequestCommand(command.ShowOne):
    """Create new zone transfer request"""

    def get_parser(self, prog_name):
        parser = super(CreateTransferRequestCommand, self).get_parser(
            prog_name)

        parser.add_argument('zone_id', help="Zone ID to transfer.",)
        parser.add_argument(
            '--target-project-id',
            help="Target Project ID to transfer to.")
        parser.add_argument('--description', help="Description")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_transfers.create_request(
            parsed_args.zone_id, parsed_args.target_project_id,
            parsed_args.description)
        return six.moves.zip(*sorted(six.iteritems(data)))


class ListTransferRequestsCommand(command.Lister):
    """List Zone Transfer Requests"""

    columns = ['id', 'zone_id', 'zone_name', 'project_id',
               'target_project_id', 'status', 'key']

    def get_parser(self, prog_name):
        parser = super(ListTransferRequestsCommand, self).get_parser(
            prog_name)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_transfers.list_requests()

        cols = self.columns
        return cols, (utils.get_item_properties(s, cols) for s in data)


class ShowTransferRequestCommand(command.ShowOne):
    """Show Zone Transfer Request Details"""

    def get_parser(self, prog_name):
        parser = super(ShowTransferRequestCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Zone Tranfer Request ID")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_transfers.get_request(parsed_args.id)

        return six.moves.zip(*sorted(six.iteritems(data)))


class SetTransferRequestCommand(command.ShowOne):
    """Set a Zone Transfer Request"""

    def get_parser(self, prog_name):
        parser = super(SetTransferRequestCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Zone Transfer Request ID")
        description_group = parser.add_mutually_exclusive_group()
        description_group.add_argument('--description', help="Description")
        description_group.add_argument('--no-description', action='store_true')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = {}

        if parsed_args.no_description:
            data['description'] = None
        elif parsed_args.description:
            data['description'] = parsed_args.description

        updated = client.zone_transfers.update_request(parsed_args.id, data)
        return six.moves.zip(*sorted(six.iteritems(updated)))


class DeleteTransferRequestCommand(command.Command):
    """Delete a Zone Transfer Request"""
    def get_parser(self, prog_name):
        parser = super(DeleteTransferRequestCommand, self).get_parser(
            prog_name)

        parser.add_argument('id', help="Zone Transfer Request ID")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        client.zone_transfers.delete_request(parsed_args.id)

        LOG.info('Zone Transfer %s was deleted', parsed_args.id)


class AcceptTransferRequestCommand(command.ShowOne):
    """Accept a Zone Transfer Request"""

    def get_parser(self, prog_name):
        parser = super(AcceptTransferRequestCommand, self).get_parser(
            prog_name)

        parser.add_argument('--transfer-id', help="Transfer ID", type=str,
                            required=True)
        parser.add_argument('--key', help="Transfer Key", type=str,
                            required=True)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_transfers.accept_request(
            parsed_args.transfer_id, parsed_args.key)
        return six.moves.zip(*sorted(six.iteritems(data)))


class ListTransferAcceptsCommand(command.Lister):
    """List Zone Transfer Accepts"""

    columns = ['id', 'zone_id', 'project_id',
               'zone_transfer_request_id', 'status', 'key']

    def get_parser(self, prog_name):
        parser = super(ListTransferAcceptsCommand, self).get_parser(
            prog_name)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_transfers.list_requests()

        cols = self.columns
        return cols, (utils.get_item_properties(s, cols) for s in data)


class ShowTransferAcceptCommand(command.ShowOne):
    """Show Zone Transfer Accept"""

    def get_parser(self, prog_name):
        parser = super(ShowTransferAcceptCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Zone Tranfer Accept ID")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_transfers.get_accept(parsed_args.id)

        return six.moves.zip(*sorted(six.iteritems(data)))


class ExportZoneCommand(command.ShowOne):
    """Export a Zone"""

    def get_parser(self, prog_name):
        parser = super(ExportZoneCommand, self).get_parser(
            prog_name)

        common.add_all_common_options(parser)

        parser.add_argument('zone_id', help="Zone ID", type=str)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_exports.create(parsed_args.zone_id)
        _format_zone_export_record(data)

        LOG.info('Zone Export %s was created', data['id'])

        return six.moves.zip(*sorted(six.iteritems(data)))


class ListZoneExportsCommand(command.Lister):
    """List Zone Exports"""

    columns = [
        'id',
        'zone_id',
        'created_at',
        'status',
    ]

    def get_parser(self, prog_name):
        parser = super(ListZoneExportsCommand, self).get_parser(
            prog_name)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_exports.list()

        cols = self.columns
        return cols, (utils.get_item_properties(s, cols)
                      for s in data['exports'])


class ShowZoneExportCommand(command.ShowOne):
    """Show a Zone Export"""

    def get_parser(self, prog_name):
        parser = super(ShowZoneExportCommand, self).get_parser(
            prog_name)

        parser.add_argument('zone_export_id', help="Zone Export ID", type=str)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_exports.get_export_record(
            parsed_args.zone_export_id)
        _format_zone_export_record(data)

        return six.moves.zip(*sorted(six.iteritems(data)))


class DeleteZoneExportCommand(command.Command):
    """Delete a Zone Export"""

    def get_parser(self, prog_name):
        parser = super(DeleteZoneExportCommand, self).get_parser(
            prog_name)

        parser.add_argument('zone_export_id', help="Zone Export ID", type=str)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        client.zone_exports.delete(parsed_args.zone_export_id)

        LOG.info('Zone Export %s was deleted', parsed_args.zone_export_id)


class ShowZoneExportFileCommand(command.ShowOne):
    """Show the zone file for the Zone Export"""

    def get_parser(self, prog_name):
        parser = super(ShowZoneExportFileCommand, self).get_parser(
            prog_name)

        parser.add_argument('zone_export_id', help="Zone Export ID", type=str)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_exports.get_export(parsed_args.zone_export_id)

        return ['data'], [data]


class ImportZoneCommand(command.ShowOne):
    """Import a Zone from a file on the filesystem"""

    def get_parser(self, prog_name):
        parser = super(ImportZoneCommand, self).get_parser(
            prog_name)

        parser.add_argument('zone_file_path',
                            help="Path to a zone file", type=str)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        with open(parsed_args.zone_file_path, 'r') as f:
            zone_file_contents = f.read()

        data = client.zone_imports.create(zone_file_contents)
        _format_zone_import_record(data)

        LOG.info('Zone Import %s was created', data['id'])

        return six.moves.zip(*sorted(six.iteritems(data)))


class ListZoneImportsCommand(command.Lister):
    """List Zone Imports"""

    columns = [
        'id',
        'zone_id',
        'created_at',
        'status',
        'message',
    ]

    def get_parser(self, prog_name):
        parser = super(ListZoneImportsCommand, self).get_parser(
            prog_name)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_imports.list()

        cols = self.columns
        return cols, (utils.get_item_properties(s, cols)
                      for s in data['imports'])


class ShowZoneImportCommand(command.ShowOne):
    """Show a Zone Import"""

    def get_parser(self, prog_name):
        parser = super(ShowZoneImportCommand, self).get_parser(
            prog_name)

        parser.add_argument('zone_import_id', help="Zone Import ID", type=str)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.zone_imports.get_import_record(
            parsed_args.zone_import_id)
        _format_zone_import_record(data)

        return six.moves.zip(*sorted(six.iteritems(data)))


class DeleteZoneImportCommand(command.Command):
    """Delete a Zone Import"""

    def get_parser(self, prog_name):
        parser = super(DeleteZoneImportCommand, self).get_parser(
            prog_name)

        parser.add_argument('zone_import_id', help="Zone Import ID", type=str)

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        client.zone_imports.delete(parsed_args.zone_import_id)

        LOG.info('Zone Import %s was deleted', parsed_args.zone_import_id)



# Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
# Author: Graham Hayes <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import six


def add_all_projects_option(parser):
    parser.add_argument(
        '--all-projects',
        default=False,
        action='store_true',
        help='Show results from all projects. Default: False'
    )


def add_edit_managed_option(parser):
    parser.add_argument(
        '--edit-managed',
        default=False,
        action='store_true',
        help='Edit resources marked as managed. Default: False'
    )


def add_sudo_project_id_option(parser):
    parser.add_argument(
        '--sudo-project-id',
        default=None,
        help='Project ID to impersonate for this command. Default: None'
    )


def add_all_common_options(parser):
    add_all_projects_option(parser)
    add_edit_managed_option(parser)
    add_sudo_project_id_option(parser)


def set_all_projects(client, value):
    client.session.all_projects = value


def set_sudo_project_id(client, value):
    client.session.sudo_project_id = value


def set_edit_managed(client, value):
    client.session.edit_managed = value


def set_all_common_headers(client, parsed_args):

    if parsed_args.all_projects is not None and \
            isinstance(parsed_args.all_projects, bool):
        set_all_projects(client, parsed_args.all_projects)

    if parsed_args.edit_managed is not None and \
            isinstance(parsed_args.edit_managed, bool):
        set_edit_managed(client, parsed_args.edit_managed)

    if parsed_args.sudo_project_id is not None and \
            isinstance(parsed_args.sudo_project_id, six.string_types):
        set_sudo_project_id(client, parsed_args.sudo_project_id)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Federico Ceratto <federico.ceratto@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1 import adapter
from keystoneauth1 import session as keystone_session

from designateclient.tests.base import TestCase
from designateclient.v2.client import Client


class TestClient(TestCase):
    def test_init(self):
        self.assertRaises(ValueError, Client)

    def test_init_with_session(self):
        session = keystone_session.Session()
        adapted = adapter.Adapter(session=session)
        client = Client(session=adapted)
        assert client.session

    def test_init_with_session_timeout(self):
        session = keystone_session.Session()
        client = Client(
            session=session,
            timeout=1)
        assert client.session.timeout == 1

    def test_init_with_auth(self):
        session = keystone_session.Session()
        client = Client(
            auth='http://127.0.0.1:22/',
            session=session)
        assert client.session.auth



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from mock import patch

from designateclient.tests import v2
from designateclient.v2 import zones


class TestLimits(v2.APIV2TestCase, v2.CrudMixin):
    @patch.object(zones.ZoneController, "list")
    def test_get(self, zones_get):
        zones_get.return_value = [{"id": "foo"}]

        ref = [{
            "hostname": "ns1.example.com.",
            "priority": 1
        }]
        parts = ["zones", "foo", "nameservers"]
        self.stub_url("GET", parts=parts, json={"nameservers": ref})

        response = self.client.nameservers.list("foo")
        self.assertEqual(ref, response)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from designateclient.tests import test_v1
from designateclient import utils
from designateclient import v1

from keystoneauth1 import session as keystone_session


class TestClient(test_v1.APIV1TestCase):
    def test_all_tenants(self):
        # Create a client with the all_tenants flag set to True
        client = v1.Client(all_tenants=True)

        # Verify this has been picked up
        self.assertTrue(client.all_tenants)

    def test_all_tenants_not_supplied(self):
        # Create a client without supplying any all_tenants flag
        client = v1.Client()

        # Verify all_tenants is False
        self.assertFalse(client.all_tenants)
        self.assertIsNotNone(client.all_tenants)

    def test_all_tenants_through_session(self):
        # Create a session with the all_tenants flag set to True
        session = utils.get_session(
            auth_url='Anything',
            endpoint='Anything',
            domain_id='Anything',
            domain_name='Anything',
            project_id='Anything',
            project_name='Anything',
            project_domain_name='Anything',
            project_domain_id='Anything',
            username='Anything',
            user_id='Anything',
            password='Anything',
            user_domain_id='Anything',
            user_domain_name='Anything',
            token=None,
            insecure=False,
            cacert=None,
            all_tenants=True)

        # Create a client using the pre-created session
        client = v1.Client(session=session)

        # Verify the all_tenants flag has been picked up
        self.assertTrue(client.all_tenants)

    def test_edit_managed(self):
        # Create a client with the edit_managed flag set to True
        client = v1.Client(edit_managed=True)

        # Verify this has been picked up
        self.assertTrue(client.edit_managed)

    def test_edit_managed_not_supplied(self):
        # Create a client without supplying any edit_managed flag
        client = v1.Client()

        # Verify edit_managed is False
        self.assertFalse(client.edit_managed)
        self.assertIsNotNone(client.edit_managed)

    def test_edit_managed_through_session(self):
        # Create a session with the edit_managed flag set to True
        session = utils.get_session(
            auth_url='Anything',
            endpoint='Anything',
            domain_id='Anything',
            domain_name='Anything',
            project_id='Anything',
            project_name='Anything',
            project_domain_name='Anything',
            project_domain_id='Anything',
            username='Anything',
            user_id='Anything',
            password='Anything',
            user_domain_id='Anything',
            user_domain_name='Anything',
            token=None,
            insecure=False,
            cacert=None,
            edit_managed=True)

        # Create a client using the pre-created session
        client = v1.Client(session=session)

        # Verify the edit_managed flag has been picked up
        self.assertTrue(client.edit_managed)

    def test_timeout_new_session(self):
        client = v1.Client(
            auth_url="http://127.0.0.1:22/",
            timeout=1,
        )
        assert client.session.timeout == 1

    def test_timeout_override_session_timeout(self):
        # The adapter timeout should override the session timeout
        session = keystone_session.Session(timeout=10)
        client = v1.Client(
            auth_url="http://127.0.0.1:22/",
            session=session,
            timeout=2,
        )
        self.assertEqual(2, client.session.timeout)



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json

from designateclient import client
from designateclient import utils
from designateclient import warlock


Domain = warlock.model_factory(utils.load_schema('v1', 'domain'))
Server = warlock.model_factory(utils.load_schema('v1', 'server'))


class DomainsController(client.CrudController):
    def list(self):
        """
        Retrieve a list of domains

        :returns: A list of :class:`Domain`
        """
        response = self.client.get('/domains')

        return [Domain(i) for i in response.json()['domains']]

    def get(self, domain_id):
        """
        Retrieve a domain

        :param domain_id: Domain Identifier
        :returns: :class:`Domain`
        """
        response = self.client.get('/domains/%s' % domain_id)

        return Domain(response.json())

    def create(self, domain):
        """
        Create a domain

        :param domain: A :class:`Domain` to create
        :returns: :class:`Domain`
        """
        response = self.client.post('/domains', data=json.dumps(domain))

        return Domain(response.json())

    def update(self, domain):
        """
        Update a domain

        :param domain: A :class:`Domain` to update
        :returns: :class:`Domain`
        """
        response = self.client.put('/domains/%s' % domain.id,
                                   data=json.dumps(domain.changes))

        return Domain(response.json())

    def delete(self, domain):
        """
        Delete a domain

        :param domain: A :class:`Domain`, or Domain Identifier to delete
        """
        if isinstance(domain, Domain):
            self.client.delete('/domains/%s' % domain.id)
        else:
            self.client.delete('/domains/%s' % domain)

    def list_domain_servers(self, domain_id):
        """
        Retrieve the list of nameservers for a domain

        :param domain_id: Domain Identifier
        :returns: A list of :class:`Server`
        """
        response = self.client.get('/domains/%s/servers' % domain_id)

        return [Server(i) for i in response.json()['servers']]



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
import logging

from osc_lib.command import command
import six

from designateclient import utils
from designateclient.v2.cli import common
from designateclient.v2.utils import get_all


LOG = logging.getLogger(__name__)


def _format_recordset(recordset):
    # Remove unneeded fields for display output formatting
    recordset['records'] = "\n".join(recordset['records'])
    recordset.pop('links', None)
    return recordset


def _has_project_id(data):
    if len(data) < 1:
        return False
    if 'project_id' in data[0]:
        return True
    return False


class ListRecordSetsCommand(command.Lister):
    """List recordsets"""

    columns = ['id', 'name', 'type', 'records', 'status', 'action']

    def get_parser(self, prog_name):
        parser = super(ListRecordSetsCommand, self).get_parser(prog_name)

        parser.add_argument('--name', help="RecordSet Name", required=False)
        parser.add_argument('--type', help="RecordSet Type", required=False)
        parser.add_argument('--data', help="RecordSet Record Data",
                            required=False)
        parser.add_argument('--ttl', help="Time To Live (Seconds)",
                            required=False)
        parser.add_argument('--description', help="Description",
                            required=False)
        parser.add_argument('--status', help="RecordSet Status",
                            required=False)
        parser.add_argument('--action', help="RecordSet Action",
                            required=False)

        parser.add_argument('zone_id', help="Zone ID. To list all"
                            " recordsets specify 'all'")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        criterion = {}
        if parsed_args.type is not None:
            criterion["type"] = parsed_args.type

        if parsed_args.name is not None:
            criterion["name"] = parsed_args.name

        if parsed_args.data is not None:
            criterion["data"] = parsed_args.data

        if parsed_args.ttl is not None:
            criterion["ttl"] = parsed_args.ttl

        if parsed_args.description is not None:
            criterion["description"] = parsed_args.description

        if parsed_args.status is not None:
            criterion["status"] = parsed_args.status

        if parsed_args.action is not None:
            criterion["action"] = parsed_args.action

        cols = self.columns

        if parsed_args.zone_id == 'all':
            data = get_all(client.recordsets.list_all_zones,
                           criterion=criterion)
            cols.insert(2, 'zone_name')
        else:
            data = get_all(client.recordsets.list, args=[parsed_args.zone_id],
                           criterion=criterion)

        if client.session.all_projects and _has_project_id(data):
            cols.insert(1, 'project_id')

        for i, rs in enumerate(data):
            data[i] = _format_recordset(rs)

        return cols, (utils.get_item_properties(s, cols) for s in data)


class ShowRecordSetCommand(command.ShowOne):
    """Show recordset details"""

    def get_parser(self, prog_name):
        parser = super(ShowRecordSetCommand, self).get_parser(prog_name)

        parser.add_argument('zone_id', help="Zone ID")
        parser.add_argument('id', help="RecordSet ID")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)
        data = client.recordsets.get(parsed_args.zone_id, parsed_args.id)

        _format_recordset(data)
        return six.moves.zip(*sorted(six.iteritems(data)))


class CreateRecordSetCommand(command.ShowOne):
    """Create new recordset"""

    log = logging.getLogger('deprecated')

    def get_parser(self, prog_name):
        parser = super(CreateRecordSetCommand, self).get_parser(prog_name)

        parser.add_argument('zone_id', help="Zone ID")
        parser.add_argument('name', help="RecordSet Name")
        req_group = parser.add_mutually_exclusive_group(required=True)
        req_group.add_argument(
            '--records',
            help=argparse.SUPPRESS,
            nargs='+')
        req_group.add_argument(
            '--record',
            help="RecordSet Record, repeat if necessary",
            action='append')
        parser.add_argument('--type', help="RecordSet Type", required=True)
        parser.add_argument('--ttl', type=int, help="Time To Live (Seconds)")
        parser.add_argument('--description', help="Description")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        all_records = parsed_args.record or parsed_args.records
        if parsed_args.records:
            self.log.warning(
                "Option --records is deprecated, use --record instead.")
        data = client.recordsets.create(
            parsed_args.zone_id,
            parsed_args.name,
            parsed_args.type,
            all_records,
            description=parsed_args.description,
            ttl=parsed_args.ttl)

        _format_recordset(data)
        return six.moves.zip(*sorted(six.iteritems(data)))


class SetRecordSetCommand(command.ShowOne):
    """Set recordset properties"""

    def get_parser(self, prog_name):
        parser = super(SetRecordSetCommand, self).get_parser(prog_name)

        parser.add_argument('zone_id', help="Zone ID")
        parser.add_argument('id', help="RecordSet ID")
        req_group = parser.add_mutually_exclusive_group()
        req_group.add_argument(
            '--records',
            help=argparse.SUPPRESS,
            nargs='+')
        req_group.add_argument(
            '--record',
            help="RecordSet Record, repeat if necessary",
            action='append')

        description_group = parser.add_mutually_exclusive_group()
        description_group.add_argument('--description', help="Description")
        description_group.add_argument('--no-description', action='store_true')

        ttl_group = parser.add_mutually_exclusive_group()
        ttl_group.add_argument('--ttl', type=int, help="TTL")
        ttl_group.add_argument('--no-ttl', action='store_true')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        data = {}

        if parsed_args.no_description:
            data['description'] = None
        elif parsed_args.description:
            data['description'] = parsed_args.description

        if parsed_args.no_ttl:
            data['ttl'] = None
        elif parsed_args.ttl:
            data['ttl'] = parsed_args.ttl

        all_records = parsed_args.record or parsed_args.records
        if parsed_args.records:
            self.log.warning(
                "Option --records is deprecated, use --record instead.")

        if all_records:
            data['records'] = all_records

        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        updated = client.recordsets.update(
            parsed_args.zone_id,
            parsed_args.id,
            data)

        _format_recordset(updated)

        return six.moves.zip(*sorted(six.iteritems(updated)))


class DeleteRecordSetCommand(command.ShowOne):
    """Delete recordset"""

    def get_parser(self, prog_name):
        parser = super(DeleteRecordSetCommand, self).get_parser(prog_name)

        parser.add_argument('zone_id', help="Zone ID")
        parser.add_argument('id', help="RecordSet ID")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)
        data = client.recordsets.delete(parsed_args.zone_id, parsed_args.id)

        LOG.info('RecordSet %s was deleted', parsed_args.id)

        _format_recordset(data)
        return six.moves.zip(*sorted(six.iteritems(data)))



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from osc_lib.command import command
import six

from designateclient import utils
from designateclient.v2.cli import common
from designateclient.v2.utils import get_all


LOG = logging.getLogger(__name__)


def _format_tld(tld):
    # Remove unneeded fields for display output formatting
    tld.pop('links', None)


class ListTLDsCommand(command.Lister):
    """List tlds"""

    columns = ['id', 'name', 'description']

    def get_parser(self, prog_name):
        parser = super(ListTLDsCommand, self).get_parser(prog_name)

        parser.add_argument('--name', help="TLD NAME")

        parser.add_argument('--description', help="TLD Description")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = get_all(client.tlds.list)

        cols = self.columns
        return cols, (utils.get_item_properties(s, cols) for s in data)


class ShowTLDCommand(command.ShowOne):
    """Show tld details"""

    def get_parser(self, prog_name):
        parser = super(ShowTLDCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="TLD ID")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)
        data = client.tlds.get(parsed_args.id)
        _format_tld(data)
        return six.moves.zip(*sorted(six.iteritems(data)))


class CreateTLDCommand(command.ShowOne):
    """Create new tld"""

    def get_parser(self, prog_name):
        parser = super(CreateTLDCommand, self).get_parser(prog_name)

        parser.add_argument('--name', help="TLD Name", required=True)
        parser.add_argument('--description', help="Description")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)
        data = client.tlds.create(parsed_args.name, parsed_args.description)
        _format_tld(data)
        return six.moves.zip(*sorted(six.iteritems(data)))


class SetTLDCommand(command.ShowOne):
    """Set tld properties"""

    def get_parser(self, prog_name):
        parser = super(SetTLDCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="TLD ID")
        parser.add_argument('--name', help="TLD Name")
        description_group = parser.add_mutually_exclusive_group()
        description_group.add_argument('--description', help="Description")
        description_group.add_argument('--no-description', action='store_true')

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        data = {}

        if parsed_args.name:
            data['name'] = parsed_args.name

        if parsed_args.no_description:
            data['description'] = None
        elif parsed_args.description:
            data['description'] = parsed_args.description

        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        data = client.tlds.update(parsed_args.id, data)
        _format_tld(data)
        return six.moves.zip(*sorted(six.iteritems(data)))


class DeleteTLDCommand(command.Command):
    """Delete tld"""

    def get_parser(self, prog_name):
        parser = super(DeleteTLDCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="TLD ID")

        common.add_all_common_options(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)
        client.tlds.delete(parsed_args.id)

        LOG.info('TLD %s was deleted', parsed_args.id)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from designateclient.tests import test_v1
from designateclient import utils
from designateclient import warlock


Record = warlock.model_factory(utils.load_schema('v1', 'record'))

DOMAIN = {
    "id": str(uuid.uuid4()),
    "name": "example.com."
}


class TestRecords(test_v1.APIV1TestCase, test_v1.CrudMixin):
    RESOURCE = 'records'

    def new_ref(self, **kwargs):
        ref = super(TestRecords, self).new_ref(**kwargs)
        ref.setdefault("name", uuid.uuid4().hex)
        ref.setdefault("type", "A")
        ref.setdefault("data", "10.0.0.1")
        return ref

    def test_create_record(self):
        ref = self.new_ref(id="2e32e609-3a4f-45ba-bdef-e50eacd345ad")

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_url("POST", parts=parts, json=ref)

        values = ref.copy()
        del values["id"]

        self.client.records.create(DOMAIN['id'], Record(values))
        self.assertRequestBodyIs(json=values)

    def test_create_AAAA_record(self):
        ref = self.new_ref(id="11112222-3333-4444-5555-666677778888",
                           type="AAAA",
                           data="2001:db8:0:1234:0:5678:9:12")

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_url("POST", parts=parts, json=ref)

        values = ref.copy()
        del values["id"]

        self.client.records.create(DOMAIN['id'], Record(values))
        self.assertRequestBodyIs(json=values)

    def test_create_MX_record(self):
        ref = self.new_ref(id="11112222-3333-4444-5555-666677778989",
                           type="MX",
                           data="mail.example.com.",
                           priority=10)

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_url("POST", parts=parts, json=ref)

        values = ref.copy()
        del values["id"]

        self.client.records.create(DOMAIN['id'], Record(values))
        self.assertRequestBodyIs(json=values)

    def test_create_CNAME_record(self):
        ref = self.new_ref(id="11112222-3333-4444-5555-666677778890",
                           type="CNAME",
                           data="example.com.")

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_url("POST", parts=parts, json=ref)

        values = ref.copy()
        del values["id"]

        self.client.records.create(DOMAIN['id'], Record(values))
        self.assertRequestBodyIs(json=values)

    def test_create_TXT_record(self):
        ref = self.new_ref(id="11112222-3333-4444-5555-666677778889",
                           type="TXT",
                           data="This is a TXT record")

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_url("POST", parts=parts, json=ref)

        values = ref.copy()
        del values["id"]

        self.client.records.create(DOMAIN['id'], Record(values))
        self.assertRequestBodyIs(json=values)

    def test_create_SRV_record(self):
        ref = self.new_ref(id="11112222-3333-4444-5555-666677778888",
                           type="SRV",
                           data="0 5060 sip.example.com.",
                           priority=30)

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_url("POST", parts=parts, json=ref)

        values = ref.copy()
        del values["id"]

        self.client.records.create(DOMAIN['id'], Record(values))
        self.assertRequestBodyIs(json=values)

    def test_create_NS_record(self):
        ref = self.new_ref(id="11112222-3333-4444-5555-666677779999",
                           type="NS",
                           data="ns1.example.com.")

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_url("POST", parts=parts, json=ref)

        values = ref.copy()
        del values["id"]

        self.client.records.create(DOMAIN['id'], Record(values))
        self.assertRequestBodyIs(json=values)

    def test_create_PTR_record(self):
        ref = self.new_ref(id="11112222-3333-4444-5555-666677778891",
                           type="PTR",
                           data="www.example.com.")

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_url("POST", parts=parts, json=ref)

        values = ref.copy()
        del values["id"]

        self.client.records.create(DOMAIN['id'], Record(values))
        self.assertRequestBodyIs(json=values)

    def test_create_SPF_record(self):
        ref = self.new_ref(id="11112222-3333-4444-5555-666677778899",
                           type="SPF",
                           data="v=spf1 +mx a:colo.example.com/28 -all")

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_url("POST", parts=parts, json=ref)

        values = ref.copy()
        del values["id"]

        self.client.records.create(DOMAIN['id'], Record(values))
        self.assertRequestBodyIs(json=values)

    def test_create_SSHFP_record(self):
        ref = self.new_ref(id="11112222-3333-4444-5555-666677778888",
                           type="SSHFP",
                           data="2 1 6c3c958af43d953f91f40e0d84157f4fe7b4a898")

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_url("POST", parts=parts, json=ref)

        values = ref.copy()
        del values["id"]

        self.client.records.create(DOMAIN['id'], Record(values))
        self.assertRequestBodyIs(json=values)

    def test_get(self):
        ref = self.new_ref(id="2e32e609-3a4f-45ba-bdef-e50eacd345ad")

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_entity("GET", entity=ref, id=ref["id"], parts=parts)

        response = self.client.records.get(DOMAIN["id"], ref["id"])
        self.assertEqual(ref, response)

    def test_list(self):
        items = [
            self.new_ref(id="2e32e609-3a4f-45ba-bdef-e50eacd345ad"),
            self.new_ref(id="11112222-3333-4444-5555-666677778888")
        ]

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_url("GET", parts=parts, json={"records": items})

        listed = self.client.records.list(DOMAIN["id"])
        self.assertList(items, listed)
        self.assertQueryStringIs("")

    def test_update(self):
        ref = self.new_ref(id="2e32e609-3a4f-45ba-bdef-e50eacd345ad",
                           type="A",
                           data="192.0.2.5")

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_entity("PUT", entity=ref, id=ref["id"], parts=parts)

        values = ref.copy()
        del values["id"]

        self.client.records.update(DOMAIN["id"], Record(ref))

    def test_delete(self):
        ref = self.new_ref()

        parts = ["domains", DOMAIN["id"], self.RESOURCE]
        self.stub_entity("DELETE", id=ref["id"], parts=parts)

        self.client.records.delete(DOMAIN["id"], ref["id"])
        self.assertRequestBodyIs(None)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designateclient.v2.base import V2Controller
from designateclient.v2 import utils as v2_utils


class ZoneController(V2Controller):
    def create(self, name, type_=None, email=None, description=None, ttl=None,
               masters=None, attributes=None):
        type_ = type_ or "PRIMARY"

        data = {
            "name": name,
            "type": type_
        }

        if type_ == "PRIMARY":
            if email:
                data["email"] = email

            if ttl is not None:
                data["ttl"] = ttl

        elif type_ == "SECONDARY" and masters:
            data["masters"] = masters

        if description is not None:
            data["description"] = description

        if attributes is not None:
            data["attributes"] = attributes

        return self._post('/zones', data=data)

    def list(self, criterion=None, marker=None, limit=None):
        url = self.build_url('/zones', criterion, marker, limit)

        return self._get(url, response_key="zones")

    def get(self, zone):
        zone = v2_utils.resolve_by_name(self.list, zone)

        return self._get('/zones/%s' % zone)

    def update(self, zone, values):
        zone = v2_utils.resolve_by_name(self.list, zone)

        url = self.build_url('/zones/%s' % zone)

        return self._patch(url, data=values)

    def delete(self, zone):
        zone = v2_utils.resolve_by_name(self.list, zone)

        url = self.build_url('/zones/%s' % zone)

        return self._delete(url)

    def abandon(self, zone):
        zone = v2_utils.resolve_by_name(self.list, zone)

        url = '/zones/%s/tasks/abandon' % zone

        self.client.session.post(url)

    def axfr(self, zone):
        zone = v2_utils.resolve_by_name(self.list, zone)

        url = '/zones/%s/tasks/xfr' % zone

        self.client.session.post(url)


class ZoneTransfersController(V2Controller):
    def create_request(self, zone, target_project_id, description=None):
        zone = v2_utils.resolve_by_name(self.client.zones.list, zone)

        data = {
            "target_project_id": target_project_id
        }

        if description is not None:
            data["description"] = description

        url = '/zones/%s/tasks/transfer_requests' % zone

        return self._post(url, data=data)

    def get_request(self, transfer_id):
        url = '/zones/tasks/transfer_requests/%s' % transfer_id
        return self._get(url)

    def list_requests(self):
        url = '/zones/tasks/transfer_requests'
        return self._get(url, response_key="transfer_requests")

    def update_request(self, transfer_id, values):
        url = '/zones/tasks/transfer_requests/%s' % transfer_id
        return self._patch(url, data=values)

    def delete_request(self, transfer_id):
        url = '/zones/tasks/transfer_requests/%s' % transfer_id
        self._delete(url)

    def accept_request(self, transfer_id, key):
        url = '/zones/tasks/transfer_accepts'

        data = {
            "key": key,
            "zone_transfer_request_id": transfer_id
        }
        return self._post(url, data=data)

    def get_accept(self, accept_id):
        url = '/zones/tasks/transfer_accepts/%s' % accept_id
        return self._get(url)

    def list_accepts(self):
        url = '/zones/tasks/transfer_accepts'
        return self._get(url, response_key="transfer_accepts")


class ZoneExportsController(V2Controller):
    def create(self, zone):
        zone_id = v2_utils.resolve_by_name(self.client.zones.list, zone)

        return self._post('/zones/%s/tasks/export' % zone_id)

    def get_export_record(self, zone_export_id):
        return self._get('/zones/tasks/exports/%s' % zone_export_id)

    def list(self):
        return self._get('/zones/tasks/exports')

    def delete(self, zone_export_id):
        return self._delete('/zones/tasks/exports/%s' % zone_export_id)

    def get_export(self, zone_export_id):
        return self._get('/zones/tasks/exports/%s/export' % zone_export_id,
                         headers={'Accept': 'text/dns'})


class ZoneImportsController(V2Controller):
    def create(self, zone_file_contents):
        return self._post('/zones/tasks/imports', data=zone_file_contents,
                          headers={'Content-Type': 'text/dns'})

    def get_import_record(self, zone_import_id):
        return self._get('/zones/tasks/imports/%s' % zone_import_id)

    def list(self):
        return self._get('/zones/tasks/imports')

    def delete(self, zone_import_id):
        return self._delete('/zones/tasks/imports/%s' % zone_import_id)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from mock import patch

from designateclient.tests import test_v1
from designateclient.v1 import quotas


class TestQuota(test_v1.APIV1TestCase, test_v1.CrudMixin):

    @patch.object(quotas.QuotasController, "get")
    def test_get(self, quota_get):
        QUOTA = {"domains": 10,
                 "recordset_records": 20,
                 "domain_records": 500,
                 "domain_recordsets": 500}
        quota_get.return_value = QUOTA
        response = self.client.quotas.get("foo")
        self.assertEqual(QUOTA, response)

    @patch.object(quotas.QuotasController, "update")
    def test_update(self, quota_update):
        args = mock.MagicMock()
        args.tenant_id = "1234"
        args.value = {"domains": 1000}
        self.client.quotas.update(args.tenant_id, args.value)
        self.client.quotas.update.assert_called_with(args.tenant_id,
                                                     args.value)

    @patch.object(quotas.QuotasController, "reset")
    def test_reset(self, quota_reset):
        args = mock.MagicMock()
        args.tenant_id = "1234"
        self.client.quotas.reset(args.tenant_id)
        self.client.quotas.reset.assert_called_with("1234")



# Copyright 2013 Hewlett-Packard Development Company, L.P. All Rights Reserved.
#
# Author: Patrick Galbraith <patg@patg.net>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designateclient import client


class ReportsController(client.Controller):
    def count_all(self):
        """
        Domain, Records and tenant total count
        """
        response = self.client.get('/reports/counts')

        return response.json()

    def count_domains(self):
        """
        Domain total count
        """
        response = self.client.get('/reports/counts/domains')

        return response.json()

    def count_tenants(self):
        """
        Tenant total count
        """
        response = self.client.get('/reports/counts/tenants')

        return response.json()

    def count_records(self):
        """
        Record total count
        """
        response = self.client.get('/reports/counts/records')

        return response.json()

    def tenants_all(self):
        """
        Per tenant count
        """
        response = self.client.get('/reports/tenants')

        return response.json()['tenants']

    def tenant_domains(self, other_tenant_id):
        """
        Tenant's domain count
        """
        response = self.client.get('/reports/tenants/%s' %
                                   other_tenant_id)

        return [{'domain': d} for d in response.json()['domains']]



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from designateclient.cli import base
from designateclient.v1.servers import Server

LOG = logging.getLogger(__name__)


class ListServersCommand(base.ListCommand):
    """List Servers"""

    columns = ['id', 'name']

    def execute(self, parsed_args):
        return self.client.servers.list()


class GetServerCommand(base.GetCommand):
    """Get Server"""

    def get_parser(self, prog_name):
        parser = super(GetServerCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Server ID.")

        return parser

    def execute(self, parsed_args):
        return self.client.servers.get(parsed_args.id)


class CreateServerCommand(base.CreateCommand):
    """Create Server"""

    def get_parser(self, prog_name):
        parser = super(CreateServerCommand, self).get_parser(prog_name)

        parser.add_argument('--name', help="Server name.", required=True)

        return parser

    def execute(self, parsed_args):
        server = Server(
            name=parsed_args.name,
        )

        return self.client.servers.create(server)


class UpdateServerCommand(base.UpdateCommand):
    """Update Server"""

    def get_parser(self, prog_name):
        parser = super(UpdateServerCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Server ID.")
        parser.add_argument('--name', help="Server name.")

        return parser

    def execute(self, parsed_args):
        # TODO(kiall): API needs updating.. this get is silly
        server = self.client.servers.get(parsed_args.id)

        if parsed_args.name:
            server.name = parsed_args.name

        return self.client.servers.update(server)


class DeleteServerCommand(base.DeleteCommand):
    """Delete Server"""

    def get_parser(self, prog_name):
        parser = super(DeleteServerCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Server ID.")

        return parser

    def execute(self, parsed_args):
        return self.client.servers.delete(parsed_args.id)



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from designateclient.cli import base


LOG = logging.getLogger(__name__)


class GetQuotaCommand(base.GetCommand):
    """Get Quota"""

    def get_parser(self, prog_name):
        parser = super(GetQuotaCommand, self).get_parser(prog_name)

        parser.add_argument('tenant_id', help="Tenant ID")

        return parser

    def execute(self, parsed_args):
        return self.client.quotas.get(parsed_args.tenant_id)


class UpdateQuotaCommand(base.UpdateCommand):
    """Update Quota"""

    def get_parser(self, prog_name):
        parser = super(UpdateQuotaCommand, self).get_parser(prog_name)

        parser.add_argument('tenant_id', help="Tenant ID.")
        parser.add_argument('--domains', help="Allowed domains.", type=int)
        parser.add_argument('--domain-recordsets',
                            help="Allowed domain records.",
                            type=int)
        parser.add_argument('--recordset-records',
                            help="Allowed recordset records.",
                            type=int)
        parser.add_argument('--domain-records',
                            help="Allowed domain records.",
                            type=int)
        parser.add_argument('--api-export-size',
                            help="Allowed zone export recordsets.",
                            type=int)
        return parser

    def execute(self, parsed_args):
        # TODO(kiall): API needs updating.. this get is silly
        quota = self.client.quotas.get(parsed_args.tenant_id)

        for key, old in quota.items():
            new = getattr(parsed_args, key)
            if new is not None and new != old:
                quota[key] = new
        return self.client.quotas.update(parsed_args.tenant_id, quota)


class ResetQuotaCommand(base.DeleteCommand):
    """Reset Quota"""

    def get_parser(self, prog_name):
        parser = super(ResetQuotaCommand, self).get_parser(prog_name)

        parser.add_argument('tenant_id', help="Tenant ID.")

        return parser

    def execute(self, parsed_args):
        self.client.quotas.reset(parsed_args.tenant_id)



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""OpenStackClient plugin for DNS service."""

from osc_lib import utils as oscutils

from designateclient import shell


DEFAULT_API_VERSION = '2'

API_NAME = 'dns'
API_VERSION_OPTION = 'os_dns_api_version'
API_VERSIONS = {
    '2': 'designateclient.v2.client.Client',
}


def make_client(instance):
    cls = oscutils.get_client_class(
        API_NAME, instance._api_version[API_NAME],
        API_VERSIONS)
    kwargs = oscutils.build_kwargs_dict('endpoint_type', instance._interface)

    return cls(session=instance.session,
               region_name=instance._region_name, **kwargs)


def build_option_parser(parser):
    """Hook to add global options."""
    parser.add_argument(
        '--os-dns-api-version',
        metavar='<dns-api-version>',
        default=shell.env('OS_DNS_API_VERSION', default="2"),
        help='DNS API version, default=' +
             DEFAULT_API_VERSION +
             ' (Env: OS_DNS_API_VERSION)')

    return parser



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_utils import uuidutils
import six

from designateclient.v2.base import V2Controller
from designateclient.v2 import utils as v2_utils


class RecordSetController(V2Controller):
    def _canonicalize_record_name(self, zone, name):
        zone_info = None

        # If we get a zone name we'll need to get the ID of it before POST.
        if isinstance(zone, six.string_types) and not \
                uuidutils.is_uuid_like(zone):
                zone_info = self.client.zones.get(zone)
        elif isinstance(zone, dict):
            zone_info = zone

        # We where given a name like "www" vs www.i.io., attempt to fix it on
        # the behalf of the actor.
        if not name.endswith("."):
            if not isinstance(zone_info, dict):
                zone_info = self.client.zones.get(zone)

            name = "%s.%s" % (name, zone_info["name"])

        return name, zone_info

    def create(self, zone, name, type_, records, description=None,
               ttl=None):
        name, zone_info = self._canonicalize_record_name(zone, name)

        data = {
            'name': name,
            'type': type_,
            'records': records
        }

        if ttl is not None:
            data['ttl'] = ttl

        if description is not None:
            data['description'] = description

        if zone_info is not None:
            zone_id = zone_info["id"]
        else:
            zone_id = zone

        url = '/zones/%s/recordsets' % zone_id
        return self._post(url, data=data)

    def list(self, zone, criterion=None, marker=None, limit=None):
        zone = v2_utils.resolve_by_name(self.client.zones.list, zone)

        url = self.build_url(
            '/zones/%s/recordsets' % zone,
            criterion, marker, limit)

        return self._get(url, response_key='recordsets')

    def list_all_zones(self, criterion=None, marker=None, limit=None):

        url = self.build_url('/recordsets', criterion, marker, limit)

        return self._get(url, response_key='recordsets')

    def get(self, zone, recordset):
        zone = v2_utils.resolve_by_name(self.client.zones.list, zone)
        recordset = v2_utils.resolve_by_name(self.list, recordset, zone)

        url = self.build_url('/zones/%s/recordsets/%s' % (
                             zone, recordset))

        return self._get(url)

    def update(self, zone, recordset, values):
        zone = v2_utils.resolve_by_name(self.client.zones.list, zone)
        recordset = v2_utils.resolve_by_name(self.list, recordset, zone)

        url = '/zones/%s/recordsets/%s' % (zone, recordset)

        return self._put(url, data=values)

    def delete(self, zone, recordset):
        zone = v2_utils.resolve_by_name(self.client.zones.list, zone)
        recordset = v2_utils.resolve_by_name(self.list, recordset, zone)

        url = '/zones/%s/recordsets/%s' % (zone, recordset)

        return self._delete(url)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from mock import patch

from designateclient.tests import test_v1
from designateclient import utils
from designateclient.v1 import domains
from designateclient import warlock

Domain = warlock.model_factory(utils.load_schema('v1', 'domain'))


class TestDomain(test_v1.APIV1TestCase, test_v1.CrudMixin):
    RESOURCE = 'domains'

    def new_ref(self, **kwargs):
        ref = super(TestDomain, self).new_ref(**kwargs)
        ref.setdefault("name", uuid.uuid4().hex)
        ref.setdefault("email", "abc@example.com.")
        ref.setdefault("ttl", 3600)
        return ref

    def test_create(self):
        ref = {"id": "89acac79-38e7-497d-807c-a011e1310438",
               "name": "domain1.com.",
               "email": "nsadmin@example.org",
               "ttl": 60}
        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        response = self.client.domains.create(values["name"])
        self.assertEqual(ref['id'], response['id'])

    def test_create_with_description(self):
        ref = {"id": "89acac79-38e7-497d-807c-a011e1310438",
               "name": "domain1.com.",
               "email": "nsadmin@example.org",
               "ttl": 60,
               "description": "fully qualified domain"}

        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        response = self.client.domains.create(values["name"])
        self.assertEqual(ref['id'], response['id'])

    def test_create_with_description_too_long(self):
        ref = {"id": "89acac79-38e7-497d-807c-a011e1310438",
               "name": "domain1.com.",
               "email": "nsadmin@example.org",
               "ttl": 60,
               "description": "d" * 161}
        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.assertRaises(ValueError, self.client.domains.create,
                          values["name"])

    def test_create_with_zero_ttl(self):
        ref = {"id": "89acac79-38e7-497d-807c-a011e1310438",
               "name": "domain1.com.",
               "email": "nsadmin@example.org",
               "ttl": 0}
        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.assertRaises(ValueError, self.client.domains.create,
                          values["name"])

    def test_create_with_negative_ttl(self):
        ref = {"id": "89acac79-38e7-497d-807c-a011e1310438",
               "name": "domain1.com.",
               "email": "nsadmin@example.org",
               "ttl": -1}
        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.assertRaises(ValueError, self.client.domains.create,
                          values["name"])

    def test_create_with_no_ttl(self):
        ref = {"id": "89acac79-38e7-497d-807c-a011e1310438",
               "name": "domain1.com.",
               "email": "nsadmin@example.org",
               "ttl": ""}
        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.assertRaises(ValueError, self.client.domains.create,
                          values["name"])

    def test_create_with_name_too_long(self):
        ref = {"id": "89acac79-38e7-497d-807c-a011e1310438",
               "name": "domain" + "a" * 255 + ".com.",
               "email": "nsadmin@example.org",
               "ttl": 60}
        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.assertRaises(ValueError, self.client.domains.create,
                          values["name"])

    def test_list(self):
        items = [
            self.new_ref(email="abc@example.org",
                         id="89acac79-38e7-497d-807c-a011e1310438"),
            self.new_ref(email="root@example.org",
                         id="89acac79-38e7-497d-807c-a011e1310435")
        ]

        self.stub_url("GET", parts=[self.RESOURCE], json={"domains": items})

        listed = self.client.domains.list()
        self.assertList(items, listed)
        self.assertQueryStringIs("")

    def test_get(self):
        ref = self.new_ref(email="abc@example.org",
                           id="89acac79-38e7-497d-807c-a011e1310438")

        self.stub_entity("GET", entity=ref, id=ref["id"])

        response = self.client.domains.get(ref["id"])
        self.assertEqual(ref, response)

    def test_delete(self):
        ref = self.new_ref(email="abc@example.org",
                           id="89acac79-38e7-497d-807c-a011e1310438")

        self.stub_entity("DELETE", entity=ref, id=ref["id"])

        self.client.domains.delete(ref["id"])
        self.assertRequestBodyIs(None)

    def test_update(self):
        ref = self.new_ref(id="89acac79-38e7-497d-807c-a011e1310438")

        self.stub_entity("PUT", entity=ref, id=ref["id"])

        values = ref.copy()

        self.client.domains.update(Domain(values))

    @patch.object(domains.DomainsController, "list_domain_servers")
    def test_list_domain_servers(self, domains_get):
        domains_get.return_value = [{"id": "foo", "name": "ns1.example.com."}]

        ref = [{
            "id": "foo",
            "name": "ns1.example.com.",
        }]
        parts = ["domains", "foo", "servers"]
        self.stub_url("GET", parts=parts, json={"servers": ref})

        response = self.client.domains.list_domain_servers("foo")
        self.assertEqual(ref, response)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designateclient.v2.base import V2Controller
from designateclient.v2 import utils as v2_utils


class NameServerController(V2Controller):
    def list(self, zone):
        zone = v2_utils.resolve_by_name(self.client.zones.list, zone)

        url = '/zones/%s/nameservers' % zone

        return self._get(url, response_key='nameservers')



# Copyright 2017 SAP SE
#
# Author: Rudolf Vriend <rudolf.vriend@sap.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from designateclient.tests import v2


class TestTSIGKeys(v2.APIV2TestCase, v2.CrudMixin):
    RESOURCE = 'tsigkeys'

    def new_ref(self, **kwargs):
        ref = super(TestTSIGKeys, self).new_ref(**kwargs)
        ref.setdefault("name", uuid.uuid4().hex)
        ref.setdefault("algorithm", "hmac-sha256")
        ref.setdefault("secret", uuid.uuid4().hex)
        ref.setdefault("scope", "POOL")
        ref.setdefault("resource_id", uuid.uuid4().hex)
        return ref

    def test_create(self):
        ref = self.new_ref()

        self.stub_url("POST", parts=[self.RESOURCE], json=ref)

        values = ref.copy()
        del values["id"]

        self.client.tsigkeys.create(**values)
        self.assertRequestBodyIs(json=values)

    def test_get(self):
        ref = self.new_ref()

        self.stub_entity("GET", entity=ref, id=ref["id"])

        response = self.client.tsigkeys.get(ref["id"])
        self.assertEqual(ref, response)

    def test_get_by_name(self):
        ref = self.new_ref(name="www")

        self.stub_entity("GET", entity=ref, id=ref["id"])
        self.stub_url("GET", parts=[self.RESOURCE], json={"tsigkeys": [ref]})

        response = self.client.tsigkeys.get(ref['name'])

        self.assertEqual("GET", self.requests.request_history[0].method)
        self.assertEqual(
            "http://127.0.0.1:9001/v2/tsigkeys?name=www",
            self.requests.request_history[0].url)

        self.assertEqual(ref, response)

    def test_list(self):
        items = [
            self.new_ref(),
            self.new_ref()
        ]

        self.stub_url("GET", parts=[self.RESOURCE], json={"tsigkeys": items})

        listed = self.client.tsigkeys.list()
        self.assertList(items, listed)
        self.assertQueryStringIs("")

    def test_update(self):
        ref = self.new_ref()

        self.stub_entity("PATCH", entity=ref, id=ref["id"])

        values = ref.copy()
        del values["id"]

        self.client.tsigkeys.update(ref["id"], values)
        self.assertRequestBodyIs(json=values)

    def test_delete(self):
        ref = self.new_ref()

        self.stub_entity("DELETE", id=ref["id"])

        self.client.tsigkeys.delete(ref["id"])
        self.assertRequestBodyIs(None)



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json

from designateclient import client


class QuotasController(client.Controller):
    def get(self, tenant_id):
        """
        Ping a service on a given host
        """
        response = self.client.get('/quotas/%s' % tenant_id)

        return response.json()

    def update(self, tenant_id, values):
        response = self.client.put('/quotas/%s' % tenant_id,
                                   data=json.dumps(values))
        return response.json()

    def reset(self, tenant_id):
        response = self.client.delete('/quotas/%s' % tenant_id)

        return response



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import os
import traceback

from cliff.app import App
from cliff.commandmanager import CommandManager

from designateclient import utils
from designateclient.version import version_info as version


def env(*vars, **kwargs):
    """Search for the first defined of possibly many env vars

    Returns the first environment variable defined in vars, or
    returns the default defined in kwargs.

    """
    for v in vars:
        value = os.environ.get(v)
        if value:
            return value
    return kwargs.get('default', '')


class DesignateShell(App):
    CONSOLE_MESSAGE_FORMAT = '%(levelname)s: %(message)s'
    DEFAULT_VERBOSE_LEVEL = 0

    def __init__(self):
        super(DesignateShell, self).__init__(
            description='Designate Client',
            version=version.version_string(),
            command_manager=CommandManager('designateclient.cli'),
        )

        self.log = logging.getLogger(__name__)

    def build_option_parser(self, description, version):
        parser = super(DesignateShell, self).build_option_parser(
            description, version)

        parser.add_argument('--os-username',
                            default=env('OS_USERNAME'),
                            help='Name used for authentication with the '
                                 'OpenStack Identity service. '
                                 'Defaults to env[OS_USERNAME].')

        parser.add_argument('--os-user-id',
                            default=env('OS_USER_ID'),
                            help='User ID used for authentication with the '
                                 'OpenStack Identity service. '
                                 'Defaults to env[OS_USER_ID].')

        parser.add_argument('--os-user-domain-id',
                            default=env('OS_USER_DOMAIN_ID'),
                            help='Defaults to env[OS_USER_DOMAIN_ID].')

        parser.add_argument('--os-user-domain-name',
                            default=env('OS_USER_DOMAIN_NAME'),
                            help='Defaults to env[OS_USER_DOMAIN_NAME].')

        parser.add_argument('--os-password',
                            default=env('OS_PASSWORD'),
                            help='Password used for authentication with the '
                                 'OpenStack Identity service. '
                                 'Defaults to env[OS_PASSWORD].')

        parser.add_argument('--os-tenant-name',
                            default=env('OS_TENANT_NAME'),
                            help='Tenant to request authorization on. '
                                 'Defaults to env[OS_TENANT_NAME].')

        parser.add_argument('--os-tenant-id',
                            default=env('OS_TENANT_ID'),
                            help='Tenant to request authorization on. '
                                 'Defaults to env[OS_TENANT_ID].')

        parser.add_argument('--os-project-name',
                            default=env('OS_PROJECT_NAME'),
                            help='Project to request authorization on. '
                                 'Defaults to env[OS_PROJECT_NAME].')

        parser.add_argument('--os-domain-name',
                            default=env('OS_DOMAIN_NAME'),
                            help='Project to request authorization on. '
                                 'Defaults to env[OS_DOMAIN_NAME].')

        parser.add_argument('--os-domain-id',
                            default=env('OS_DOMAIN_ID'),
                            help='Defaults to env[OS_DOMAIN_ID].')

        parser.add_argument('--os-project-id',
                            default=env('OS_PROJECT_ID'),
                            help='Project to request authorization on. '
                                 'Defaults to env[OS_PROJECT_ID].')

        parser.add_argument('--os-project-domain-id',
                            default=env('OS_PROJECT_DOMAIN_ID'),
                            help='Defaults to env[OS_PROJECT_DOMAIN_ID].')

        parser.add_argument('--os-project-domain-name',
                            default=env('OS_PROJECT_DOMAIN_NAME'),
                            help='Defaults to env[OS_PROJECT_DOMAIN_NAME].')

        parser.add_argument('--os-auth-url',
                            default=env('OS_AUTH_URL'),
                            help='Specify the Identity endpoint to use for '
                                 'authentication. '
                                 'Defaults to env[OS_AUTH_URL].')

        parser.add_argument('--os-region-name',
                            default=env('OS_REGION_NAME'),
                            help='Specify the region to use. '
                                 'Defaults to env[OS_REGION_NAME].')

        parser.add_argument('--os-token',
                            default=env('OS_SERVICE_TOKEN'),
                            help='Specify an existing token to use instead of '
                                 'retrieving one via authentication (e.g. '
                                 'with username & password). '
                                 'Defaults to env[OS_SERVICE_TOKEN].')

        parser.add_argument('--os-endpoint',
                            default=env('OS_DNS_ENDPOINT',
                                        'OS_SERVICE_ENDPOINT'),
                            help='Specify an endpoint to use instead of '
                                 'retrieving one from the service catalog '
                                 '(via authentication). '
                                 'Defaults to env[OS_DNS_ENDPOINT].')

        parser.add_argument('--os-endpoint-type',
                            default=env('OS_ENDPOINT_TYPE',
                                        default='publicURL'),
                            help='Defaults to env[OS_ENDPOINT_TYPE].')

        parser.add_argument('--os-service-type',
                            default=env('OS_DNS_SERVICE_TYPE', default='dns'),
                            help=("Defaults to env[OS_DNS_SERVICE_TYPE], or "
                                  "'dns'."))

        parser.add_argument('--os-cacert',
                            default=env('OS_CACERT'),
                            help=('CA certificate bundle file. Defaults to '
                                  'env[OS_CACERT].'))

        parser.add_argument('--insecure', action='store_true',
                            help="Explicitly allow 'insecure' SSL requests.")

        parser.add_argument('--all-tenants', action='store_true',
                            help="Allows to list all domains from all "
                            "tenants.")

        parser.add_argument('--edit-managed', action='store_true',
                            help='Allows to edit records that are marked as '
                            'managed.')

        return parser

    def configure_logging(self):
        """Configure logging for the app

        Cliff sets some defaults we don't want so re-work it a bit
        """

        if self.options.debug:
            # --debug forces verbose_level 3
            # Set this here so cliff.app.configure_logging() can work
            self.options.verbose_level = 3

        super(DesignateShell, self).configure_logging()
        root_logger = logging.getLogger('')

        # Requests logs some stuff at INFO that we don't want
        # unless we have DEBUG
        requests_log = logging.getLogger("requests")
        requests_log.setLevel(logging.ERROR)

        # Other modules we don't want DEBUG output for so
        # don't reset them below
        iso8601_log = logging.getLogger("iso8601")
        iso8601_log.setLevel(logging.ERROR)

        # Set logging to the requested level
        self.dump_stack_trace = False
        if self.options.verbose_level == 0:
            # --quiet
            root_logger.setLevel(logging.ERROR)
        elif self.options.verbose_level == 1:
            # This is the default case, no --debug, --verbose or --quiet
            root_logger.setLevel(logging.WARNING)
        elif self.options.verbose_level == 2:
            # One --verbose
            root_logger.setLevel(logging.INFO)
        elif self.options.verbose_level >= 3:
            # Two or more --verbose
            root_logger.setLevel(logging.DEBUG)
            requests_log.setLevel(logging.DEBUG)

        if self.options.debug:
            # --debug forces traceback
            self.dump_stack_trace = True

    def initialize_app(self, argv):
        super(DesignateShell, self).initialize_app(argv)
        self.session = utils.get_session(
            auth_url=self.options.os_auth_url,
            endpoint=self.options.os_endpoint,
            domain_id=self.options.os_domain_id,
            domain_name=self.options.os_domain_name,
            project_id=self.options.os_project_id or self.options.os_tenant_id,
            project_name=(self.options.os_project_name or
                          self.options.os_tenant_name),
            project_domain_name=self.options.os_project_domain_name,
            project_domain_id=self.options.os_project_domain_id,
            username=self.options.os_username,
            user_id=self.options.os_user_id,
            password=self.options.os_password,
            user_domain_id=self.options.os_user_domain_id,
            user_domain_name=self.options.os_user_domain_name,
            token=self.options.os_token,
            insecure=self.options.insecure,
            cacert=self.options.os_cacert
        )

    def run(self, argv):
        try:
            return super(DesignateShell, self).run(argv)
        except Exception as e:
            if not logging.getLogger('').handlers:
                logging.basicConfig()
            if self.dump_stack_trace:
                self.log.error(traceback.format_exc(e))
            else:
                self.log.error('Exception raised: ' + str(e))
            return 1



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from designateclient.cli import base
from designateclient.v1.records import Record

LOG = logging.getLogger(__name__)


class ListRecordsCommand(base.ListCommand):
    """List Records"""

    columns = ['id', 'type', 'name', 'data']

    def get_parser(self, prog_name):
        parser = super(ListRecordsCommand, self).get_parser(prog_name)

        parser.add_argument('domain_id', help="Domain ID or name.")

        return parser

    def execute(self, parsed_args):
        domain_id = self.find_resourceid_by_name_or_id(
            'domains', parsed_args.domain_id)
        return self.client.records.list(domain_id)


class GetRecordCommand(base.GetCommand):
    """Get Record"""

    def get_parser(self, prog_name):
        parser = super(GetRecordCommand, self).get_parser(prog_name)

        parser.add_argument('domain_id', help="Domain ID or name.")
        parser.add_argument('id', help="Record ID.")

        return parser

    def execute(self, parsed_args):
        domain_id = self.find_resourceid_by_name_or_id(
            'domains', parsed_args.domain_id)
        return self.client.records.get(domain_id, parsed_args.id)


class CreateRecordCommand(base.CreateCommand):
    """Create Record"""

    def get_parser(self, prog_name):
        parser = super(CreateRecordCommand, self).get_parser(prog_name)

        parser.add_argument('domain_id', help="Domain ID or name.")
        parser.add_argument(
            '--name', help="Record (relative|absolute) name.", required=True)
        parser.add_argument('--type', help="Record type.", required=True)
        parser.add_argument('--data', help="Record data.", required=True)
        parser.add_argument('--ttl', type=int, help="Record TTL.")
        parser.add_argument('--priority', type=int, help="Record priority.")
        parser.add_argument('--description', help="Description.")

        return parser

    def execute(self, parsed_args):
        domain_id = self.find_resourceid_by_name_or_id(
            'domains', parsed_args.domain_id)

        if not parsed_args.name.endswith('.'):
            # Relative name?
            domain_name = self.client.domains.get(domain_id)['name']
            absolute = parsed_args.name + '.'
            relative = absolute + domain_name
            if absolute.endswith('.' + domain_name):
                # Relative name or absolute name missing final period?
                msg = ('"%s" is a relative name but looks like an absolute '
                       'name, use --name "%s" or "%s"'
                       % (parsed_args.name, absolute, relative))
                raise ValueError(msg)
            parsed_args.name = relative

        record = Record(
            name=parsed_args.name,
            type=parsed_args.type,
            data=parsed_args.data,
        )

        if parsed_args.ttl is not None:
            record.ttl = parsed_args.ttl

        if parsed_args.priority is not None:
            record.priority = parsed_args.priority

        if parsed_args.description:
            record.description = parsed_args.description

        return self.client.records.create(domain_id, record)


class UpdateRecordCommand(base.UpdateCommand):
    """Update Record"""

    def get_parser(self, prog_name):
        parser = super(UpdateRecordCommand, self).get_parser(prog_name)

        parser.add_argument('domain_id', help="Domain ID or name.")
        parser.add_argument('id', help="Record ID.")
        parser.add_argument('--name', help="Record name.")
        parser.add_argument('--type', help="Record type.")
        parser.add_argument('--data', help="Record data.")

        description_group = parser.add_mutually_exclusive_group()
        description_group.add_argument('--description', help="Description.")
        description_group.add_argument('--no-description', action='store_true')

        ttl_group = parser.add_mutually_exclusive_group()
        ttl_group.add_argument('--ttl', type=int,
                               help="Record time to live (seconds).")
        ttl_group.add_argument('--no-ttl', action='store_true')

        priotity_group = parser.add_mutually_exclusive_group()
        priotity_group.add_argument('--priority', type=int,
                                    help="Record priority.")
        priotity_group.add_argument('--no-priority', action='store_true')

        return parser

    def execute(self, parsed_args):
        # TODO(kiall): API needs updating.. this get is silly
        record = self.client.records.get(parsed_args.domain_id, parsed_args.id)

        if parsed_args.name:
            record.name = parsed_args.name

        if parsed_args.type:
            record.type = parsed_args.type

        if parsed_args.data:
            record.data = parsed_args.data

        if parsed_args.no_ttl:
            record.ttl = None
        elif parsed_args.ttl is not None:
            record.ttl = parsed_args.ttl

        if parsed_args.no_priority:
            record.priority = None
        elif parsed_args.priority is not None:
            record.priority = parsed_args.priority

        if parsed_args.no_description:
            record.description = None
        elif parsed_args.description:
            record.description = parsed_args.description

        domain_id = self.find_resourceid_by_name_or_id(
            'domains', parsed_args.domain_id)
        return self.client.records.update(domain_id, record)


class DeleteRecordCommand(base.DeleteCommand):
    """Delete Record"""

    def get_parser(self, prog_name):
        parser = super(DeleteRecordCommand, self).get_parser(prog_name)

        parser.add_argument('domain_id', help="Domain ID or name.")
        parser.add_argument('id', help="Record ID.")

        return parser

    def execute(self, parsed_args):
        domain_id = self.find_resourceid_by_name_or_id(
            'domains', parsed_args.domain_id)
        return self.client.records.delete(domain_id, parsed_args.id)



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from designateclient.cli import base
from designateclient.v1.domains import Domain

LOG = logging.getLogger(__name__)


class ListDomainsCommand(base.ListCommand):
    """List Domains"""

    columns = ['id', 'name', 'serial']

    def execute(self, parsed_args):
        return self.client.domains.list()


class GetDomainCommand(base.GetCommand):
    """Get Domain"""

    def get_parser(self, prog_name):
        parser = super(GetDomainCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Domain ID or name.")

        return parser

    def execute(self, parsed_args):
        id = self.find_resourceid_by_name_or_id('domains', parsed_args.id)
        return self.client.domains.get(id)


class CreateDomainCommand(base.CreateCommand):
    """Create Domain"""

    def get_parser(self, prog_name):
        parser = super(CreateDomainCommand, self).get_parser(prog_name)

        parser.add_argument('--name', help="Domain name.", required=True)
        parser.add_argument('--email', help="Domain email.", required=True)
        parser.add_argument('--ttl', type=int, help="Time to live (seconds).")
        parser.add_argument('--description', help="Description.")

        return parser

    def execute(self, parsed_args):
        domain = Domain(
            name=parsed_args.name,
            email=parsed_args.email,
        )

        if parsed_args.description:
            domain.description = parsed_args.description

        if parsed_args.ttl is not None:
            domain.ttl = parsed_args.ttl

        return self.client.domains.create(domain)


class UpdateDomainCommand(base.UpdateCommand):
    """Update Domain"""

    def get_parser(self, prog_name):
        parser = super(UpdateDomainCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Domain ID or name.")
        parser.add_argument('--name', help="Domain name.")
        parser.add_argument('--email', help="Domain email.")
        parser.add_argument('--ttl', type=int, help="Time to live (seconds).")
        description_group = parser.add_mutually_exclusive_group()
        description_group.add_argument('--description', help="Description.")
        description_group.add_argument('--no-description', action='store_true')

        return parser

    def execute(self, parsed_args):
        # TODO(kiall): API needs updating.. this get is silly
        id = self.find_resourceid_by_name_or_id('domains', parsed_args.id)
        domain = self.client.domains.get(id)

        if parsed_args.name:
            domain.name = parsed_args.name

        if parsed_args.email:
            domain.email = parsed_args.email

        if parsed_args.ttl is not None:
            domain.ttl = parsed_args.ttl

        if parsed_args.no_description:
            domain.description = None
        elif parsed_args.description:
            domain.description = parsed_args.description

        return self.client.domains.update(domain)


class DeleteDomainCommand(base.DeleteCommand):
    """Delete Domain"""

    def get_parser(self, prog_name):
        parser = super(DeleteDomainCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Domain ID or name.")

        return parser

    def execute(self, parsed_args):
        id = self.find_resourceid_by_name_or_id('domains', parsed_args.id)
        return self.client.domains.delete(id)


class ListDomainServersCommand(base.ListCommand):
    """List Domain Servers"""

    columns = ['name']

    def get_parser(self, prog_name):
        parser = super(ListDomainServersCommand, self).get_parser(prog_name)

        parser.add_argument('id', help="Domain ID or name.")

        return parser

    def execute(self, parsed_args):
        id = self.find_resourceid_by_name_or_id('domains', parsed_args.id)
        return self.client.domains.list_domain_servers(id)



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import itertools
import logging

from cliff import command
from cliff import show
import six

from designateclient.v2.cli import common

LOG = logging.getLogger(__name__)


DNS_QUOTAS = {
    "api_export_size": "api-export-size",
    "recordset_records": "recordset-records",
    "zone_records": "zone-records",
    "zone_recordsets": "zone-recordsets",
    "zones": "zones"
}


class ListQuotasCommand(show.ShowOne):
    """List quotas"""

    # columns = ['resource', 'hard_limit']

    def get_parser(self, prog_name):
        parser = super(ListQuotasCommand, self).get_parser(prog_name)

        common.add_all_common_options(parser)

        parser.add_argument(
            '--project-id',
            help="Project ID Default: current project")

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        proj_id = parsed_args.project_id or client.session.get_project_id()

        if parsed_args.project_id != client.session.get_project_id():
            common.set_all_projects(client, True)

        data = client.quotas.list(proj_id)
        return six.moves.zip(*sorted(six.iteritems(data)))


class SetQuotasCommand(show.ShowOne):
    """Set quotas"""

    def _build_options_list(self):
            return itertools.chain(DNS_QUOTAS.items())

    def get_parser(self, prog_name):
        parser = super(SetQuotasCommand, self).get_parser(prog_name)

        common.add_all_common_options(parser)

        parser.add_argument('--project-id', help="Project ID")
        for k, v in self._build_options_list():
            parser.add_argument(
                '--%s' % v,
                metavar='<%s>' % v,
                dest=k,
                type=int,
                help='New value for the %s quota' % v,
            )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        quotas = {}
        for k, v in DNS_QUOTAS.items():
            value = getattr(parsed_args, k, None)
            if value is not None:
                quotas[k] = value

        proj_id = parsed_args.project_id or client.session.get_project_id()

        if parsed_args.project_id != client.session.get_project_id():
            common.set_all_projects(client, True)

        updated = client.quotas.update(proj_id, quotas)

        return six.moves.zip(*sorted(six.iteritems(updated)))


class ResetQuotasCommand(command.Command):
    """Reset quotas"""

    def get_parser(self, prog_name):
        parser = super(ResetQuotasCommand, self).get_parser(prog_name)

        common.add_all_common_options(parser)

        parser.add_argument('--project-id', help="Project ID")

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns
        common.set_all_common_headers(client, parsed_args)

        proj_id = parsed_args.project_id or client.session.get_project_id()

        if parsed_args.project_id != client.session.get_project_id():
            common.set_all_projects(client, True)

        client.quotas.reset(proj_id)

        LOG.info('Quota for project %s was reset', parsed_args.project_id)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from designateclient.tests import base


class CrudMixin(object):
    path_prefix = None

    def new_ref(self, **kwargs):
        kwargs.setdefault('id', uuid.uuid4().hex)
        return kwargs

    def stub_entity(self, method, parts=None, entity=None, id=None, **kwargs):
        if entity:
            kwargs['json'] = entity

        if not parts:
            parts = [self.RESOURCE]

            if self.path_prefix:
                parts.insert(0, self.path_prefix)

        if id:
            if not parts:
                parts = []

            parts.append(id)

        self.stub_url(method, parts=parts, **kwargs)

    def assertList(self, expected, actual):
        self.assertEqual(len(expected), len(actual))
        for i in expected:
            self.assertIn(i, actual)


class APIV1TestCase(base.APITestCase):
    VERSION = "1"



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import abc
import warnings

from keystoneauth1 import exceptions as ks_exceptions
from osc_lib.command import command
import six

from designateclient import exceptions
from designateclient import utils
from designateclient.v1 import Client


@six.add_metaclass(abc.ABCMeta)
class Command(command.Command):
    def run(self, parsed_args):

        warnings.simplefilter('once', category=DeprecationWarning)
        warnings.warn(
            'The "designate" CLI is being deprecated in favour of the '
            '"openstack" CLI plugin. All designate API v2 commands are '
            'implemented there. When the v1 API is removed this CLI will '
            'stop functioning',
            DeprecationWarning)
        warnings.resetwarnings()
        warnings.simplefilter('ignore', category=DeprecationWarning)

        self.client = Client(
            region_name=self.app.options.os_region_name,
            service_type=self.app.options.os_service_type,
            endpoint_type=self.app.options.os_endpoint_type,
            session=self.app.session,
            all_tenants=self.app.options.all_tenants,
            edit_managed=self.app.options.edit_managed,
            endpoint=self.app.options.os_endpoint)
        warnings.resetwarnings()
        try:
            return super(Command, self).run(parsed_args)
        except exceptions.RemoteError as e:
            columns = ['Code', 'Type']
            values = [e.code, e.type]

            if e.message:
                columns.append('Message')
                values.append(e.message)

            if e.errors:
                columns.append('Errors')
                values.append(e.errors)

            self.error_output(parsed_args, columns, values)
        except ks_exceptions.EndpointNotFound as e:
            self.app.log.error('No endpoint was found. You must provide a '
                               'username or user id via --os-username, '
                               '--os-user-id, env[OS_USERNAME] or '
                               'env[OS_USER_ID]. You may also be using a '
                               'cloud that does not have the V1 API enabled. '
                               'If your cloud does not have the V1 DNS API '
                               'use the openstack CLI to interact with the '
                               'DNS Service.')

            return 1

    def error_output(self, parsed_args, column_names, data):
        self.formatter.emit_one(column_names,
                                data,
                                self.app.stdout,
                                parsed_args)
        self.app.log.error('The requested action did not complete '
                           'successfully')

    @abc.abstractmethod
    def execute(self, parsed_args):
        """
        Execute something, this is since we overload self.take_action()
        in order to format the data

        This method __NEEDS__ to be overloaded!

        :param parsed_args: The parsed args that are given by take_action()
        """

    def post_execute(self, data):
        """
        Format the results locally if needed, by default we just return data

        :param data: Whatever is returned by self.execute()
        """
        return data

    def take_action(self, parsed_args):
        results = self.execute(parsed_args)
        return self.post_execute(results)

    def find_resourceid_by_name_or_id(self, resource_plural, name_or_id):
        resource_client = getattr(self.client, resource_plural)
        return utils.find_resourceid_by_name_or_id(resource_client, name_or_id)


class ListCommand(Command, command.Lister):
    columns = None

    def post_execute(self, results):
        if len(results) > 0:
            columns = self.columns or utils.get_columns(results)
            data = [utils.get_item_properties(i, columns) for i in results]
            return columns, data
        else:
            return [], ()


class GetCommand(Command, command.ShowOne):
    def post_execute(self, results):
        return results.keys(), results.values()


class CreateCommand(Command, command.ShowOne):
    def post_execute(self, results):
        return results.keys(), results.values()


class UpdateCommand(Command, command.ShowOne):
    def post_execute(self, results):
        return results.keys(), results.values()


class DeleteCommand(Command, command.ShowOne):
    def post_execute(self, results):
        return [], []



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from mock import patch

from designateclient.tests import test_v1
from designateclient.v1 import reports


class TestReports(test_v1.APIV1TestCase, test_v1.CrudMixin):

    @patch.object(reports.ReportsController, "count_all")
    def test_count_all(self, count_all):
        self.client.reports.count_all()
        self.client.reports.count_all.assert_called_with()

    @patch.object(reports.ReportsController, "count_domains")
    def test_count_domain(self, count_domains):
        self.client.reports.count_domains()
        self.client.reports.count_domains.assert_called_once_with()

    @patch.object(reports.ReportsController, "count_tenants")
    def test_count_tenants(self, count_tenants):
        self.client.reports.count_tenants()
        self.client.reports.count_tenants.assert_called_once_with()

    @patch.object(reports.ReportsController, "count_records")
    def test_count_records(self, count_records):
        self.client.reports.count_records()
        self.client.reports.count_records.assert_called_once_with()

    @patch.object(reports.ReportsController, "tenants_all")
    def test_tenants_all(self, tenants_all):
        self.client.reports.tenants_all()
        self.client.reports.tenants_all.assert_called_once_with()

    @patch.object(reports.ReportsController, "tenant_domains")
    def test_tenant_domains(self, tenant_domains):
        args = mock.MagicMock()
        args.other_tenant_id = "uuid"
        self.client.reports.tenant_domains(args.other_tenant_id)
        self.client.reports.tenant_domains.assert_called_once_with("uuid")



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Federico Ceratto <federico.ceratto@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1.identity import generic
from keystoneauth1 import session as keystone_session
from mock import Mock

from designateclient.tests import v2
from designateclient.v2.client import Client


def create_session(timeout=None):
    auth = generic.Password(auth_url='', username='', password='',
                            tenant_name='')
    return keystone_session.Session(auth=auth, timeout=timeout)


class TestTimeout(v2.APIV2TestCase, v2.CrudMixin):

    def setUp(self):
        super(TestTimeout, self).setUp()

        # Mock methods in KeyStone's Session
        self._saved_methods = (
            keystone_session.Session.get_auth_headers,
            keystone_session.Session.get_endpoint,
            keystone_session.Session._send_request,
        )

        resp = Mock()
        resp.text = ''
        resp.status_code = 200

        keystone_session.Session.get_auth_headers = Mock(
            return_value=[]
        )
        keystone_session.Session.get_endpoint = Mock(
            return_value='foo'
        )
        keystone_session.Session._send_request = Mock(
            return_value=resp,
        )
        self.mock_send_request = keystone_session.Session._send_request

    def tearDown(self):
        super(TestTimeout, self).tearDown()
        (
            keystone_session.Session.get_auth_headers,
            keystone_session.Session.get_endpoint,
            keystone_session.Session._send_request,
        ) = self._saved_methods

    def _call_request_and_check_timeout(self, client, timeout):
        """call the mocked _send_request() and check if the timeout was set
        """
        client.limits.get()
        self.assertTrue(self.mock_send_request.called)
        kw = self.mock_send_request.call_args[1]
        if timeout is None:
            self.assertNotIn('timeout', kw)
        else:
            self.assertEqual(timeout, kw['timeout'])

    def test_no_timeout(self):
        session = create_session(timeout=None)
        client = Client(session=session)
        self.assertIsNone(session.timeout)
        self.assertIsNone(client.session.timeout)
        self._call_request_and_check_timeout(client, None)

    def test_timeout_in_session(self):
        session = create_session(timeout=1)
        client = Client(session=session)
        self.assertEqual(1, session.timeout)
        self.assertIsNone(client.session.timeout)
        self._call_request_and_check_timeout(client, 1)

    def test_timeout_override_session_timeout(self):
        # The adapter timeout should override the session timeout
        session = create_session(timeout=10)
        self.assertEqual(10, session.timeout)
        client = Client(session=session, timeout=2)
        self.assertEqual(2, client.session.timeout)
        self._call_request_and_check_timeout(client, 2)

    def test_timeout_update(self):
        session = create_session(timeout=1)
        client = Client(session=session)
        self.assertEqual(1, session.timeout)
        self.assertIsNone(client.session.timeout)
        self._call_request_and_check_timeout(client, 1)

        session.timeout = 2
        self.assertEqual(2, session.timeout)

        self._call_request_and_check_timeout(client, 2)



from __future__ import print_function

import logging

from designateclient import exceptions
from designateclient import shell
from designateclient.v2 import client

from keystoneauth1.identity import generic
from keystoneauth1 import session as keystone_session


logging.basicConfig(level='DEBUG')

auth = generic.Password(
    auth_url=shell.env('OS_AUTH_URL'),
    username=shell.env('OS_USERNAME'),
    password=shell.env('OS_PASSWORD'),
    project_name=shell.env('OS_PROJECT_NAME'),
    project_domain_id='default',
    user_domain_id='default')

session = keystone_session.Session(auth=auth)

client = client.Client(session=session)


try:
    zone = client.zones.create('i.io.', email='i@i.io')
except exceptions.RemoteError:
    zone = dict([(z['name'], z) for z in client.zones.list()])['i.io.']

print(client.recordsets.list(zone['id']))



# Copyright 2013 Hewlett-Packard Development Company, L.P. All Rights Reserved.
#
# Author: Patrick Galbraith <patg@patg.net>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designateclient.cli import base


class DomainCountCommand(base.GetCommand):
    """Get counts for total domains"""

    def execute(self, parsed_args):
        return self.client.reports.count_domains()


class RecordCountCommand(base.GetCommand):
    """Get counts for total records"""

    def execute(self, parsed_args):
        return self.client.reports.count_records()


class TenantCountCommand(base.GetCommand):
    """Get counts for total tenants"""

    def execute(self, parsed_args):
        return self.client.reports.count_tenants()


class CountsCommand(base.GetCommand):
    """Get count totals for all tenants, domains and records"""

    def execute(self, parsed_args):
        return self.client.reports.count_all()


class TenantsCommand(base.ListCommand):
    """Get list of tenants and domain count for each"""

    columns = ['domain_count', 'id']

    def execute(self, parsed_args):
        return self.client.reports.tenants_all()


class TenantCommand(base.ListCommand):
    """Get a list of domains for given tenant"""

    columns = ['domain']

    def get_parser(self, prog_name):
        parser = super(TenantCommand, self).get_parser(prog_name)

        parser.add_argument('--report-tenant-id',
                            help="The tenant_id being reported on.",
                            required=True)

        return parser

    def execute(self, parsed_args):
        return self.client.reports.tenant_domains(parsed_args.report_tenant_id)



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from designateclient import client
from designateclient import utils
from designateclient.v1.domains import Domain
from designateclient import warlock


Record = warlock.model_factory(utils.load_schema('v1', 'record'))


class RecordsController(client.CrudController):
    def list(self, domain):
        """
        Retrieve a list of records

        :param domain: :class:`Domain` or Domain Identifier
        :returns: A list of :class:`Record`
        """
        domain_id = domain.id if isinstance(domain, Domain) else domain

        response = self.client.get('/domains/%(domain_id)s/records' % {
            'domain_id': domain_id
        })

        return [Record(i) for i in response.json()['records']]

    def get(self, domain, record_id):
        """
        Retrieve a record

        :param domain: :class:`Domain` or Domain Identifier
        :param record_id: Record Identifier
        :returns: :class:`Record`
        """
        domain_id = domain.id if isinstance(domain, Domain) else domain

        uri = '/domains/%(domain_id)s/records/%(record_id)s' % {
            'domain_id': domain_id,
            'record_id': record_id
        }

        response = self.client.get(uri)

        return Record(response.json())

    def create(self, domain, record):
        """
        Create a record

        :param domain: :class:`Domain` or Domain Identifier
        :param record: A :class:`Record` to create
        :returns: :class:`Record`
        """
        domain_id = domain.id if isinstance(domain, Domain) else domain

        uri = '/domains/%(domain_id)s/records' % {
            'domain_id': domain_id
        }

        response = self.client.post(uri, data=json.dumps(record))

        return Record(response.json())

    def update(self, domain, record):
        """
        Update a record

        :param domain: :class:`Domain` or Domain Identifier
        :param record: A :class:`Record` to update
        :returns: :class:`Record`
        """
        domain_id = domain.id if isinstance(domain, Domain) else domain

        uri = '/domains/%(domain_id)s/records/%(record_id)s' % {
            'domain_id': domain_id,
            'record_id': record.id
        }

        response = self.client.put(uri, data=json.dumps(record.changes))

        return Record(response.json())

    def delete(self, domain, record):
        """
        Delete a record

        :param domain: :class:`Domain` or Domain Identifier
        :param record: A :class:`Record`, or Record Identifier to delete
        """
        domain_id = domain.id if isinstance(domain, Domain) else domain
        record_id = record.id if isinstance(record, Record) else record

        uri = '/domains/%(domain_id)s/records/%(record_id)s' % {
            'domain_id': domain_id,
            'record_id': record_id
        }

        self.client.delete(uri)



"""
Copyright 2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from tempest.lib.exceptions import CommandFailed

from designateclient.functionaltests.base import BaseDesignateTest
from designateclient.functionaltests.datagen import random_blacklist
from designateclient.functionaltests.v2.fixtures import BlacklistFixture


class TestBlacklist(BaseDesignateTest):

    def setUp(self):
        super(TestBlacklist, self).setUp()
        pattern = random_blacklist()
        self.blacklist = self.useFixture(BlacklistFixture(
            pattern=pattern,
            description='A random blacklist',
        )).blacklist

        self.assertEqual(self.blacklist.pattern, pattern)
        self.assertEqual(self.blacklist.description, 'A random blacklist')

    def test_zone_blacklist_list(self):
        blacklists = self.clients.as_user('admin').zone_blacklist_list()
        self.assertGreater(len(blacklists), 0)

    def test_zone_blacklist_create_and_show(self):
        client = self.clients.as_user('admin')
        blacklist = client.zone_blacklist_show(self.blacklist.id)

        self.assertEqual(self.blacklist.created_at, blacklist.created_at)
        self.assertEqual(self.blacklist.description, blacklist.description)
        self.assertEqual(self.blacklist.id, blacklist.id)
        self.assertEqual(self.blacklist.pattern, blacklist.pattern)
        self.assertEqual(self.blacklist.updated_at, blacklist.updated_at)

    def test_zone_blacklist_delete(self):
        client = self.clients.as_user('admin')
        client.zone_blacklist_delete(self.blacklist.id)
        self.assertRaises(CommandFailed, client.zone_blacklist_show,
                          self.blacklist.id)

    def test_zone_blacklist_set(self):
        client = self.clients.as_user('admin')
        updated_pattern = random_blacklist('updatedblacklist')
        blacklist = client.zone_blacklist_set(
            id=self.blacklist.id,
            pattern=updated_pattern,
            description='An updated blacklist',
        )

        self.assertEqual(blacklist.created_at, self.blacklist.created_at)
        self.assertEqual(blacklist.description, 'An updated blacklist')
        self.assertEqual(blacklist.id, self.blacklist.id)
        self.assertEqual(blacklist.pattern, updated_pattern)
        self.assertNotEqual(blacklist.updated_at, self.blacklist.updated_at)

    def test_zone_blacklist_set_no_description(self):
        client = self.clients.as_user('admin')
        blacklist = client.zone_blacklist_set(
            id=self.blacklist.id,
            no_description=True,
        )
        self.assertEqual(blacklist.description, 'None')

    def test_cannot_set_description_with_no_description_flag(self):
        client = self.clients.as_user('admin')
        self.assertRaises(CommandFailed, client.zone_blacklist_set,
                          self.blacklist.id,
                          pattern=random_blacklist(),
                          description='new description',
                          no_description=True)


class TestBlacklistNegative(BaseDesignateTest):

    def test_invalid_blacklist_command(self):
        client = self.clients.as_user('admin')
        cmd = 'zone blacklist notacommand'
        self.assertRaises(CommandFailed, client.openstack, cmd)

    def test_blacklist_create_invalid_flag(self):
        client = self.clients.as_user('admin')
        cmd = 'zone blacklist create --pattern helloworld --notaflag invalid'
        self.assertRaises(CommandFailed, client.openstack, cmd)



"""
Copyright 2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import unittest

from tempest.lib.exceptions import CommandFailed

from designateclient.functionaltests.base import BaseDesignateTest
from designateclient.functionaltests.client import DesignateCLI
from designateclient.functionaltests.datagen import random_zone_name
from designateclient.functionaltests.v2.fixtures import TransferRequestFixture
from designateclient.functionaltests.v2.fixtures import ZoneFixture


class TestZoneTransferRequest(BaseDesignateTest):

    def setUp(self):
        super(TestZoneTransferRequest, self).setUp()
        self.ensure_tld_exists('com')
        fixture = self.useFixture(ZoneFixture(
            name=random_zone_name(),
            email='test@example.com',
        ))
        self.zone = fixture.zone

    def test_list_zone_transfer_request(self):
        self.useFixture(TransferRequestFixture(self.zone))
        xfrs = self.clients.zone_transfer_request_list()
        self.assertGreater(len(xfrs), 0)

    def test_create_and_show_zone_transfer_request(self):
        transfer_request = self.useFixture(TransferRequestFixture(
            zone=self.zone,
            user='default',
            target_user='alt',
        )).transfer_request

        fetched_xfr = self.clients.zone_transfer_request_show(
            transfer_request.id)

        self.assertEqual(fetched_xfr.created_at, transfer_request.created_at)
        self.assertEqual(fetched_xfr.description, transfer_request.description)
        self.assertEqual(fetched_xfr.id, transfer_request.id)
        self.assertEqual(fetched_xfr.key, transfer_request.key)
        self.assertEqual(fetched_xfr.links, transfer_request.links)
        self.assertEqual(fetched_xfr.target_project_id,
                         transfer_request.target_project_id)
        self.assertEqual(fetched_xfr.updated_at, transfer_request.updated_at)
        self.assertEqual(fetched_xfr.status, transfer_request.status)
        self.assertEqual(fetched_xfr.zone_id, self.zone.id)
        self.assertEqual(fetched_xfr.zone_name, self.zone.name)

    def test_delete_zone_transfer_request(self):
        transfer_request = self.useFixture(TransferRequestFixture(
            zone=self.zone,
            user='default',
            target_user='alt',
        )).transfer_request

        self.clients.zone_transfer_request_delete(transfer_request.id)
        self.assertRaises(CommandFailed,
                          self.clients.zone_transfer_request_show,
                          transfer_request.id)

    @unittest.skip("Fails because `zone transfer request set` returns nothing")
    def test_set_zone_transfer_request(self):
        transfer_request = self.useFixture(TransferRequestFixture(
            zone=self.zone,
            description="old description",
        )).transfer_request

        self.assertEqual(transfer_request.description, "old description")

        updated_xfr = self.clients.zone_transfer_request_set(
            transfer_request.id,
            description="updated description")
        self.assertEqual(updated_xfr.description, "updated description")


class TestZoneTransferAccept(BaseDesignateTest):

    def setUp(self):
        super(TestZoneTransferAccept, self).setUp()
        self.ensure_tld_exists('com')
        fixture = self.useFixture(ZoneFixture(
            name=random_zone_name(),
            email='test@example.com',
        ))
        self.zone = fixture.zone

        self.target_client = DesignateCLI.as_user('alt')
        fixture = self.useFixture(TransferRequestFixture(
            zone=self.zone,
            user='default',
            target_user='alt',
            target_project_id=self.target_client.project_id,
        ))
        self.transfer_request = fixture.transfer_request

    def test_zone_transfer_accept_request(self):
        self.target_client.zone_transfer_accept_request(
            id=self.transfer_request.id,
            key=self.transfer_request.key,
        )
        self.target_client.zone_show(self.zone.id)
        self.assertRaises(CommandFailed, self.clients.zone_show, self.zone.id)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from mock import patch

from designateclient.tests import test_v1
from designateclient.v1 import diagnostics


class TestDiagnostics(test_v1.APIV1TestCase, test_v1.CrudMixin):

    @patch.object(diagnostics.DiagnosticsController, "ping")
    def test_ping(self, ping):
        args = mock.MagicMock()
        args.service = "foo"
        args.host = "host1"
        self.client.diagnostics.ping(args.host, args.service)
        self.client.diagnostics.ping.assert_called_with("host1", "foo")



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import logging

from designateclient.cli import base

LOG = logging.getLogger(__name__)


class SyncAllCommand(base.DeleteCommand):
    """Sync Everything"""

    def execute(self, parsed_args):
        self.client.sync.sync_all()

        LOG.info('Synchronization of all domains scheduled')


class SyncDomainCommand(base.DeleteCommand):
    """Sync a single Domain"""

    def get_parser(self, prog_name):
        parser = super(SyncDomainCommand, self).get_parser(prog_name)

        parser.add_argument('domain_id', help="Domain ID")

        return parser

    def execute(self, parsed_args):
        self.client.sync.sync_domain(parsed_args.domain_id)

        LOG.info('Synchronization of domain scheduled')


class SyncRecordCommand(base.DeleteCommand):
    """Sync a single Record"""

    def get_parser(self, prog_name):
        parser = super(SyncRecordCommand, self).get_parser(prog_name)

        parser.add_argument('domain_id', help="Domain ID")
        parser.add_argument('record_id', help="Record ID")

        return parser

    def execute(self, parsed_args):
        self.client.sync.sync_record(parsed_args.domain_id,
                                     parsed_args.record_id)

        LOG.info('Synchronization of record scheduled')



# Copyright 2012 Brian Waldon
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
# Code copied from Warlock, as warlock depends on jsonschema==0.2
# Hopefully we can upstream the changes ASAP.
#

import copy
import logging

import jsonschema
import six

LOG = logging.getLogger(__name__)


class InvalidOperation(RuntimeError):
    pass


class ValidationError(ValueError):
    pass


def model_factory(schema):
    """Generate a model class based on the provided JSON Schema

    :param schema: dict representing valid JSON schema
    """
    schema = copy.deepcopy(schema)

    def validator(obj):
        """Apply a JSON schema to an object"""
        try:
            jsonschema.validate(obj, schema, cls=jsonschema.Draft3Validator)
        except jsonschema.ValidationError as e:
            raise ValidationError(str(e))

    class Model(dict):
        """Self-validating model for arbitrary objects"""

        def __init__(self, *args, **kwargs):
            d = dict(*args, **kwargs)

            # we overload setattr so set this manually
            self.__dict__['validator'] = validator
            try:
                self.validator(d)
            except ValidationError as e:
                raise ValueError('Validation Error: %s' % str(e))
            else:
                dict.__init__(self, d)

            self.__dict__['changes'] = {}

        def __getattr__(self, key):
            try:
                return self.__getitem__(key)
            except KeyError:
                raise AttributeError(key)

        def __setitem__(self, key, value):
            mutation = dict(self.items())
            mutation[key] = value
            try:
                self.validator(mutation)
            except ValidationError as e:
                raise InvalidOperation(str(e))

            dict.__setitem__(self, key, value)

            self.__dict__['changes'][key] = value

        def __setattr__(self, key, value):
            self.__setitem__(key, value)

        def clear(self):
            raise InvalidOperation()

        def pop(self, key, default=None):
            raise InvalidOperation()

        def popitem(self):
            raise InvalidOperation()

        def __delitem__(self, key):
            raise InvalidOperation()

        # NOTE(termie): This is kind of the opposite of what copy usually does
        def copy(self):
            return copy.deepcopy(dict(self))

        def update(self, other):
            # NOTE(kiall): It seems update() doesn't update the
            #              self.__dict__['changes'] dict correctly.
            mutation = dict(self.items())
            mutation.update(other)
            try:
                self.validator(mutation)
            except ValidationError as e:
                raise InvalidOperation(str(e))
            dict.update(self, other)

        def iteritems(self):
            return six.iteritems(copy.deepcopy(dict(self)))

        def items(self):
            return list(six.iteritems(copy.deepcopy(dict(self))))

        def itervalues(self):
            return six.itervalues(copy.deepcopy(dict(self)))

        def keys(self):
            return list(six.iterkeys(copy.deepcopy(dict(self))))

        def values(self):
            return list(six.itervalues(copy.deepcopy(dict(self))))

        @property
        def changes(self):
            return copy.deepcopy(self.__dict__['changes'])

    Model.__name__ = str(schema['title'])
    return Model
